# -*- coding: utf-8 -*-
from __future__ import with_statement, print_function, absolute_import
import os
from requests_oauthlib import OAuth1Session


def create_oauth_token(expiration=None, scope=None, key=None, secret=None, name=None, output=True):
    """
    Script to obtain an OAuth token from Trello.

    Must have TRELLO_API_KEY and TRELLO_API_SECRET set in your environment
    To set the token's expiration, set TRELLO_EXPIRATION as a string in your
    environment settings (eg. 'never'), otherwise it will default to 30 days.

    More info on token scope here:
        https://trello.com/docs/gettingstarted/#getting-a-token-from-a-user
    """
    request_token_url = 'https://trello.com/1/OAuthGetRequestToken'
    authorize_url = 'https://trello.com/1/OAuthAuthorizeToken'
    access_token_url = 'https://trello.com/1/OAuthGetAccessToken'

    expiration = expiration or os.environ.get('TRELLO_EXPIRATION', "30days")
    scope = scope or os.environ.get('TRELLO_SCOPE', 'read,write')
    trello_key = key or os.environ['TRELLO_API_KEY']
    trello_secret = secret or os.environ['TRELLO_API_SECRET']
    name = name or os.environ.get('TRELLO_NAME', 'py-trello')

    # Step 1: Get a request token. This is a temporary token that is used for
    # having the user authorize an access token and to sign the request to obtain
    # said access token.

    session = OAuth1Session(client_key=trello_key, client_secret=trello_secret)
    response = session.fetch_request_token(request_token_url)
    resource_owner_key, resource_owner_secret = response.get('oauth_token'), response.get('oauth_token_secret')

    if output:
        print("Request Token:")
        print("    - oauth_token        = %s" % resource_owner_key)
        print("    - oauth_token_secret = %s" % resource_owner_secret)
        print("")

    # Step 2: Redirect to the provider. Since this is a CLI script we do not
    # redirect. In a web application you would redirect the user to the URL
    # below.

    print("Go to the following link in your browser:")
    print("{authorize_url}?oauth_token={oauth_token}&scope={scope}&expiration={expiration}&name={name}".format(
        authorize_url=authorize_url,
        oauth_token=resource_owner_key,
        expiration=expiration,
        scope=scope,
        name=name
    ))

    # After the user has granted access to you, the consumer, the provider will
    # redirect you to whatever URL you have told them to redirect to. You can
    # usually define this in the oauth_callback argument as well.

    # Python 3 compatibility (raw_input was renamed to input)
    try:
        inputFunc = raw_input
    except NameError:
        inputFunc = input

    accepted = 'n'
    while accepted.lower() == 'n':
        accepted = inputFunc('Have you authorized me? (y/n) ')
    oauth_verifier = inputFunc('What is the PIN? ')

    # Step 3: Once the consumer has redirected the user back to the oauth_callback
    # URL you can request the access token the user has approved. You use the
    # request token to sign this request. After this is done you throw away the
    # request token and use the access token returned. You should store this
    # access token somewhere safe, like a database, for future use.
    session = OAuth1Session(client_key=trello_key, client_secret=trello_secret,
                            resource_owner_key=resource_owner_key, resource_owner_secret=resource_owner_secret,
                            verifier=oauth_verifier)
    access_token = session.fetch_access_token(access_token_url)

    if output:
        print("Access Token:")
        print("    - oauth_token        = %s" % access_token['oauth_token'])
        print("    - oauth_token_secret = %s" % access_token['oauth_token_secret'])
        print("")
        print("You may now access protected resources using the access tokens above.")
        print("")

    return access_token

# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4



# -*- coding: utf-8 -*-
from __future__ import with_statement, print_function, absolute_import

from trello import TrelloBase
# from trello.member import Member
# from trello.card import Card
from trello.compat import force_str
# from trello.trellolist import List
# from trello.label import Label
# from trello.checklist import Checklist
# from trello.customfield import CustomFieldDefinition
from dateutil import parser as dateparser


class PowerUp(TrelloBase):
	"""
	Class representing a Trello Power-Up.
	"""

	def __init__(self,client=None,board=None, power_up_id=None, name='',author=''):
		"""
		Valid values: calendar, cardAging, recap, voting
		:board_id: ID for the board
		"""
		super(PowerUp, self).__init__()
		if board is None:
			self.client = client
		else:
			self.client = board.client
		self.id = power_up_id
		self.board = board
		self.name = name
				

	@classmethod
	def from_json(cls, trello_client=None,power_up_id=None, board=None, json_obj=None):
		"""
		Deserialize the board json object to a Board object

		:trello_client: the trello client
		:json_obj: the powerup json object

		Alternative contrustraction:

		Deserialize the powerup json object to a powerup object
		:json_obj: the json powerup object
		"""
		if json_obj is None:
			power_up = PowerUp(client=trello_client)
		else:
			power_up = PowerUp(power_up_id=json_obj['id'], name=json_obj.get('name',''))
		
		power_up.id_plugin = json_obj.get('idPlugin','')
		power_up.author = json_obj.get('author','')
		power_up.description = json_obj.get('description', '')
		power_up.public = json_obj.get('public','')
		power_up.url = json_obj.get('url','')
		power_up.organization_owner_id = json_obj.get('idOrganizationOwner','')
		power_up.overview = json_obj.get('overview','')
		power_up.json = json_obj
		return power_up
	def __repr__(self):
		return force_str(u'<Power Up %s>' % self.name)



# -*- coding: utf-8 -*-


class ResourceUnavailable(Exception):
    """Exception representing a failed request to a resource"""

    def __init__(self, msg, http_response):
        Exception.__init__(self)
        self._msg = msg
        self._status = http_response.status_code

    def __str__(self):
        return "%s (HTTP status: %s)" % (self._msg, self._status)


class Unauthorized(ResourceUnavailable):
    pass


class TokenError(Exception):
    pass



# -*- coding: utf-8 -*-
from __future__ import with_statement, print_function, absolute_import

from trello import TrelloBase
from trello.compat import force_str


class List(TrelloBase):
    """
    Class representing a Trello list. List attributes are stored on the object,
    but access to sub-objects (Cards) require an API call
    """

    def __init__(self, board, list_id, name=''):
        """Constructor

        :board: reference to the parent board
        :list_id: ID for this list
        """
        super(List, self).__init__()
        self.board = board
        self.client = board.client
        self.id = list_id
        self.name = name
        self.closed = None
        self.pos = None
        self.subscribed = None

    @classmethod
    def from_json(cls, board, json_obj):
        """
        Deserialize the list json object to a List object

        :board: the board object that the list belongs to
        :json_obj: the json list object
        """
        list = List(board, json_obj['id'], name=json_obj['name'])
        list.closed = json_obj['closed']
        list.pos = json_obj['pos']
        #this method is also called from board.py with a different json object, so we need to make sure 'subscribed' is there
        if 'subscribed' in json_obj:
            list.subscribed = json_obj['subscribed']
        return list

    def __repr__(self):
        return force_str(u'<List %s>' % self.name)

    def fetch(self):
        """Fetch all attributes for this list"""
        json_obj = self.client.fetch_json('/lists/' + self.id)
        self.name = json_obj['name']
        self.closed = json_obj['closed']
        self.pos = json_obj['pos']
        if 'subscribed' in json_obj:
            self.subscribed = json_obj['subscribed']

    def list_cards(self, card_filter="open", actions=None, query={}):
        """Lists all cards in this list"""
        query_params = query
        if card_filter:
            query_params['filter'] = card_filter
        if actions:
            query_params['actions'] = actions
        query_params['customFieldItems'] = 'true'
        json_obj = self.client.fetch_json('/lists/' + self.id + '/cards',
                                          query_params=query_params)
        return [Card.from_json(self, c) for c in json_obj]

    def list_cards_iter(self, card_filter="open", actions=None, query=None, limit=None, batch=300):
        """see https://trello.com/c/8MJOLSCs/10-limit-actions-for-cards-requests"""
        query = {} if query is None else query
        before = None
        total = 0
        while True:
            query['limit'] = batch
            if before:
                query['before'] = before
            cards = self.list_cards(card_filter=card_filter, actions=actions, query=query)
            n = len(cards)
            if n == 0:
                break
            if limit:
                cards = cards[:limit-total]
            for c in cards:
                yield c
            total += n
            before = '%x' % min([int(c.id, 16) for c in cards])
            if limit and limit <= total:
                break

    def add_card(self, name, desc=None, labels=None, due="null", source=None, position=None, assign=None, keep_from_source="all", url_source=None):
        """Add a card to this list

        :name: name for the card
        :desc: the description of the card
        :labels: a list of label IDs to be added
        :due: due date for the card
        :source: card ID from which to clone from
        :position: position of the card in the list. Must be "top", "bottom" or a positive number.
        :keep_from_source: can be used with source parameter. Can be "attachments", "checklists", "comments", "due", "labels", "members", "stickers" or "all".
        :return: the card
        """
        labels_str = ""
        if labels:
            for label in labels:
                labels_str += label.id + ","

        members_str = ""
        if assign:
            for assignee in assign:
                members_str += assignee.id + ","

        post_args = {
            'name': name,
            'idList': self.id,
            'desc': desc,
            'idLabels': labels_str[:-1],
            'due': due,
            'idMembers': members_str[:-1],
            'idCardSource': source,
            'keepFromSource': keep_from_source if source else None,
            'urlSource': url_source
        }
        if position is not None:
            post_args["pos"] = position

        json_obj = self.client.fetch_json(
            '/cards',
            http_method='POST',
            post_args=post_args)
        return Card.from_json(self, json_obj)

    def archive_all_cards(self):
        self.client.fetch_json(
            '/lists/' + self.id + '/archiveAllCards',
            http_method='POST')

    def move_all_cards(self, destination_list):
        """
        Move all cards of this list to another list.
        The list can be in the same board (or not).
        """

        self.client.fetch_json(
            '/lists/' + self.id + '/moveAllCards',
            http_method='POST',
            post_args = {
                "idBoard": destination_list.board.id,
                "idList": destination_list.id,
            })

    def fetch_actions(self, action_filter):
        """
        Fetch actions for this list can give more argv to action_filter,
        split for ',' json_obj is list
        """
        json_obj = self.client.fetch_json(
            '/lists/' + self.id + '/actions',
            query_params={'filter': action_filter})
        self.actions = json_obj
        return self.actions

    def _set_remote_attribute(self, attribute, value):
        self.client.fetch_json(
            '/lists/' + self.id + '/' + attribute,
            http_method='PUT',
            post_args={'value': value, }, )

    def close(self):
        self.client.fetch_json(
            '/lists/' + self.id + '/closed',
            http_method='PUT',
            post_args={'value': 'true', }, )
        self.closed = True

    def open(self):
        self.client.fetch_json(
            '/lists/' + self.id + '/closed',
            http_method='PUT',
            post_args={'value': 'false', }, )
        self.closed = False

    # Move this list
    def move(self, position):
        self.client.fetch_json(
            '/lists/' + self.id + '/pos',
            http_method='PUT',
            post_args={'value': position, }, )
        self.pos = position

    # Move this list to another board, with optional position
    def move_to_board(self, board, position=None):
        self.client.fetch_json(
            '/lists/' + self.id + '/idBoard',
            http_method='PUT',
            post_args={'value': board.id, }, )
        self.board = board
        if position is not None:
            self.move(position)

    # Subscribe to this list
    def subscribe(self):
        self.client.fetch_json(
        '/lists/' + self.id + '/subscribed',
        http_method='PUT',
            post_args={'value': 'true', }, )
        self.subscribed = True

    # Unsubscribe from this list
    def unsubscribe(self):
        self.client.fetch_json(
        '/lists/' + self.id + '/subscribed',
        http_method='PUT',
            post_args={'value': 'false', }, )
        self.subscribed = False

    def cardsCnt(self):
        return len(self.list_cards())

    # Change the name of the list
    def set_name(self, name):
        self.client.fetch_json(
            '/lists/{list_id}/name'.format(list_id=self.id),
            http_method='PUT',
            post_args={'value': name})
        self.name = name

    # Change the position of the list
    def set_pos(self, position):
        self.move(position)

from trello.card import Card



#!/usr/bin/python
from __future__ import with_statement, print_function
import os
import unittest
from datetime import datetime
from trello import TrelloClient, Unauthorized, ResourceUnavailable


class TrelloClientTestCase(unittest.TestCase):
    """

    Tests for TrelloClient API. Note these test are in order to
    preserve dependencies, as an API integration cannot be tested
    independently.

    """

    def setUp(self):
        self._trello = TrelloClient(os.environ['TRELLO_API_KEY'],
                                    token=os.environ['TRELLO_TOKEN'])

    def test01_list_boards(self):
        self.assertEqual(
            len(self._trello.list_boards(board_filter="open")),
            int(os.environ['TRELLO_TEST_BOARD_COUNT']))

    def test10_board_attrs(self):
        boards = self._trello.list_boards()
        for b in boards:
            self.assertIsNotNone(b.id, msg="id not provided")
            self.assertIsNotNone(b.name, msg="name not provided")
            self.assertIsNotNone(b.description, msg="description not provided")
            self.assertIsNotNone(b.closed, msg="closed not provided")
            self.assertIsNotNone(b.url, msg="url not provided")

    def test20_board_all_lists(self):
        boards = self._trello.list_boards()
        for b in boards:
            try:
                b.all_lists()
            except Exception:
                self.fail("Caught Exception getting lists")

    def test21_board_open_lists(self):
        boards = self._trello.list_boards()
        for b in boards:
            try:
                b.open_lists()
            except Exception:
                self.fail("Caught Exception getting open lists")

    def test22_board_closed_lists(self):
        boards = self._trello.list_boards()
        for b in boards:
            try:
                b.closed_lists()
            except Exception:
                self.fail("Caught Exception getting closed lists")

    def test30_list_attrs(self):
        boards = self._trello.list_boards()
        for b in boards:
            for l in b.all_lists():
                self.assertIsNotNone(l.id, msg="id not provided")
                self.assertIsNotNone(l.name, msg="name not provided")
                self.assertIsNotNone(l.closed, msg="closed not provided")
            break  # only need to test one board's lists

    def test50_list_cards(self):
        boards = self._trello.list_boards()
        for b in boards:
            for l in b.all_lists():
                for c in l.list_cards():
                    self.assertIsNotNone(c.id, msg="id not provided")
                    self.assertIsNotNone(c.name, msg="name not provided")
                    self.assertIsNotNone(c.description,
                                         msg="description not provided")
                    self.assertIsNotNone(c.closed, msg="closed not provided")
                    self.assertIsNotNone(c.url, msg="url not provided")
                break
            break
        pass

    def test51_fetch_cards(self):
        """
        Tests fetching all attributes for all cards
        """
        boards = self._trello.list_boards()
        for b in boards:
            for l in b.all_lists():
                for c in l.list_cards():
                    c.fetch()

                    self.assertIsInstance(c.date_last_activity, datetime,
                                          msg='date not provided')
                    self.assertTrue(len(c.board_id) > 0,
                                    msg='board id not provided')
                break
            break
        pass

    def test52_list_hooks(self):
        self.assertIsInstance(self._trello.list_hooks(), list)

    def test53_unauthorized(self):
        client = TrelloClient('a')
        self.assertRaises(Unauthorized,
                          client.list_boards)

    def test54_resource_unavailable(self):
        self.assertRaises(ResourceUnavailable,
                          self._trello.get_card, '0')

    def test_list_stars(self):
        """
        Test trello client star list
        """
        self.assertEqual(len(self._trello.list_stars()), int(os.environ["TRELLO_TEST_STAR_COUNT"]), "Number of stars does not match TRELLO_TEST_STAR_COUNT")

    def test_add_delete_star(self):
        """
        Test add and delete star to/from test board
        """
        test_board_id = self._trello.search(os.environ["TRELLO_TEST_BOARD_NAME"])[0].id
        new_star = self._trello.add_star(test_board_id)
        star_list = self._trello.list_stars()
        self.assertTrue(new_star in star_list, "Star id was not added in list of starred boards")
        deleted_star = self._trello.delete_star(new_star)
        star_list = self._trello.list_stars()
        self.assertFalse(deleted_star in star_list, "Star id was not deleted from list of starred boards")

class TrelloClientTestCaseWithoutOAuth(unittest.TestCase):
    """

    Tests for TrelloClient API when OAuth not activated.

    """

    def setUp(self):
        self._trello = TrelloClient(os.environ['TRELLO_API_KEY'],
                                    api_secret=os.environ['TRELLO_TOKEN'])

    def test01_oauth_not_activated(self):
        self.assertIsNone(self._trello.oauth)


def suite():
    test_classes_to_run = [TrelloClientTestCase,
                           TrelloClientTestCaseWithoutOAuth]

    loader = unittest.TestLoader()

    suites_list = []
    for test_class in test_classes_to_run:
        suite = loader.loadTestsFromTestCase(test_class)
        suites_list.append(suite)

    return unittest.TestSuite(suites_list)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import with_statement, print_function, absolute_import

from trello import TrelloBase
from trello.compat import force_str


class Checklist(TrelloBase):
    """
    Class representing a Trello checklist.
    """

    def __init__(self, client, obj, trello_card=None):
        """
        :client: Trello client object
        :obj: List of checklists on the card, sorted by position
        :trello_card: ID of the Trello card
        """
        super(Checklist, self).__init__()
        self.client = client
        self.trello_card = trello_card
        self.id = obj['id']
        self.name = obj['name']
        self.items = sorted(obj['checkItems'], key=lambda items: items.get('pos'))
        for i in self.items:
            i['checked'] = i['state'] == "complete"

    def add_checklist_item(self, name, checked=False):
        """Add a checklist item to this checklist

        :name: name of the checklist item
        :checked: True if item state should be checked, False otherwise
        :return: the checklist item json object
        """
        json_obj = self.client.fetch_json(
            '/checklists/' + self.id + '/checkItems',
            http_method='POST',
            post_args={'name': name, 'checked': checked}, )
        json_obj['checked'] = checked
        self.items.append(json_obj)
        return json_obj

    def delete_checklist_item(self, item_or_name):
        """Delete an item on this checklist

        :item_or_name: name or item object of the checklist item to delete 
        """
        ix = self._get_item_index(item_or_name)
        if ix is None:
            return

        self.client.fetch_json(
            '/checklists/'+ self.id +
            '/checkItems/'+ self.items[ix]['id'],
            http_method='DELETE')
        del self.items[ix]

    def clear(self):
        """Clear checklist by removing all checklist items"""
        # copy item list to prevent modifying while iterating, which would break
        # for-loops behaviour
        old_items = items[:] 
        for item in old_items:
            self.delete_checklist_item(item)


    def set_checklist_item(self, name, checked):
        """Set the state of an item on this checklist

        :name: name of the checklist item
        :checked: True if item state should be checked, False otherwise
        """
        ix = self._get_item_index(name)
        if ix is None:
            return

        json_obj = self.client.fetch_json(
            '/cards/' + self.trello_card +
            '/checklist/' + self.id +
            '/checkItem/' + self.items[ix]['id'],
            http_method='PUT',
            post_args={'state': 'complete' if checked else 'incomplete'})

        json_obj['checked'] = checked
        self.items[ix] = json_obj
        return json_obj
    def set_checklist_item_due(self,checklist_item, due):
        """Set the state of an item on this checklist
        :checklist_item: the checklist item object
        :due: due date in isoformat
        """
        
        datestr = due.isoformat()
        json_obj = self.client.fetch_json(
            '/cards/' + self.trello_card +
            '/checklist/' + self.id +
            '/checkItem/' + checklist_item['id'],
            http_method='PUT',
            post_args={'due': datestr})

        ix = self._get_item_index(checklist_item['name'])
        if ix is None:
            return
        self.items[ix] = json_obj
        return json_obj
    def set_checklist_item_member(self,checklist_item, member):
        """Set the state of an item on this checklist
        :checklist_item: the checklist item object
        :member: the member to assign
        """
        json_obj = self.client.fetch_json(
            '/cards/' + self.trello_card +
            '/checklist/' + self.id +
            '/checkItem/' + checklist_item['id'],
            http_method='PUT',
            post_args={'idMember': member.id})

        ix = self._get_item_index(checklist_item['name'])
        if ix is None:
            return
        self.items[ix] = json_obj
        return json_obj
    def _set_remote_attribute(self,name, attribute, value):
        ix = self._get_item_index(name)
        if ix is None:
            return
        self.client.fetch_json(
            '/cards/' + self.trello_card +
            '/checklist/' + self.id +
            '/checkItem/' + self.items[ix]['id'] + attribute,
            http_method='PUT',
            post_args={'due': value}, )
    def rename(self, new_name):
        """Rename this checklist

        :new_name: new name of the checklist
        """

        json_obj = self.client.fetch_json(
            '/checklists/' + self.id + '/name/',
            http_method='PUT',
            post_args={'value': new_name})

        self.name = json_obj['name']

        return json_obj

    def rename_checklist_item(self, name, new_name):
        """Rename the item on this checklist

        :name: name of the checklist item
        :new_name: new name of item
        """
        ix = self._get_item_index(name)
        if ix is None:
            return

        json_obj = self.client.fetch_json(
                '/cards/' + self.trello_card +
                '/checklist/' + self.id +
                '/checkItem/' + self.items[ix]['id'],
                http_method='PUT',
                post_args={'name': new_name})

        self.items[ix] = json_obj
        return json_obj

    def delete(self):
        """Removes this checklist"""
        self.client.fetch_json(
            '/checklists/%s' % self.id,
            http_method='DELETE')

    def _get_item_index(self, item_or_name):
        """Locate the index of the checklist item"""
        try:
            ix = [i for i, item in enumerate(self.items) if
                    item['name'] == item_or_name or item == item_or_name][0]
            return ix
        except ValueError:
            return None

    def __repr__(self):
        return force_str(u'<Checklist %s>' % self.id)



# -*- coding: utf-8 -*-
from __future__ import with_statement, print_function, absolute_import

from trello import TrelloBase
from trello.compat import force_str


class Label(TrelloBase):
    """
    Class representing a Trello Label.
    """
    def __init__(self, client, label_id, name, color=""):
        super(Label, self).__init__()
        self.client = client
        self.id = label_id
        self.name = name
        self.color = color

    @classmethod
    def from_json(cls, board, json_obj):
        """
        Deserialize the label json object to a Label object

        :board: the parent board the label is on
        :json_obj: the label json object
        """
        label = Label(board.client,
                      label_id=json_obj['id'],
                      name=json_obj['name'],
                      color=json_obj['color'])
        return label

    @classmethod
    def from_json_list(cls, board, json_objs):
        return [cls.from_json(board, obj) for obj in json_objs]

    def __repr__(self):
        return force_str(u'<Label %s>' % self.name)

    def fetch(self):
        """Fetch all attributes for this label"""
        json_obj = self.client.fetch_json('/labels/' + self.id)
        self.name = json_obj['name']
        self.color = json_obj['color']
        return self



# -*- coding: utf-8 -*-
from dateutil import parser as dateparser

from trello.base import TrelloBase


class Attachments(TrelloBase):
    """
    https://developers.trello.com/advanced-reference/card#get-1-cards-card-id-or-shortlink-attachments
    """
    def __init__(self, id, bytes, date, edge_color, idMember, is_upload, mime_type, name, previews, url):
        super(Attachments, self).__init__()
        self.id = id
        self.bytes = bytes
        self.date = dateparser.parse(date)
        self.edge_color = edge_color
        self.idMember = idMember
        self.is_upload = is_upload
        self.mime_type = mime_type
        self.name = name
        self.previews = previews
        self.url = url

    @staticmethod
    def from_json(json_obj):
        id = json_obj.get("id")
        bytes = json_obj.get("bytes")
        date = json_obj.get("date")
        edge_color = json_obj.get("edgeColor")
        idMember = json_obj.get("idMember")
        is_upload = json_obj.get("isUpload")
        mime_type = json_obj.get("mimeType")
        name = json_obj.get("name")
        previews = [AttachmentsPreview.from_json(preview_json) for preview_json in json_obj.get("previews")]
        url = json_obj.get("url")

        return Attachments(id, bytes, date, edge_color, idMember, is_upload, mime_type, name, previews, url)

    def __repr__(self):
        return u"<Attachments {0}>".format(self.name)


class AttachmentsPreview(object):
    def __init__(self, bytes, url, width, height, is_scaled):
        self.bytes = bytes
        self.url = url
        self.width = width
        self.height = height
        self.is_scaled = is_scaled

    @staticmethod
    def from_json(json_obj):
        bytes = json_obj.get("bytes")
        url = json_obj.get("url")
        width = json_obj.get("width")
        height = json_obj.get("height")
        is_scaled = json_obj.get("scaled")

        return AttachmentsPreview(bytes, url, width, height, is_scaled)

    def __repr__(self):
        return u"<Attachments Preview {0}x{1}".format(self.width, self.height)



#!/usr/bin/python
from __future__ import with_statement, print_function
from datetime import datetime, timedelta
import unittest
import os
from trello import TrelloClient, ResourceUnavailable


class TrelloCardTestCase(unittest.TestCase):
    """
    Tests for TrelloClient API. Note these test are in order to
    preserve dependencies, as an API integration cannot be tested
    independently.
    """

    @classmethod
    def setUpClass(cls):
        cls._trello = TrelloClient(os.environ['TRELLO_API_KEY'],
                                   token=os.environ['TRELLO_TOKEN'])
        for b in cls._trello.list_boards():
            if b.name == os.environ['TRELLO_TEST_BOARD_NAME']:
                cls._board = b
                break
        if not cls._board:
            cls.fail("Couldn't find test board")
        cls._list = cls._board.add_list(str(datetime.now()))

    def _add_card(self, name, description=None):
        try:
            card = self._list.add_card(name, description)
            self.assertIsNotNone(card, msg="card is None")
            self.assertIsNotNone(card.id, msg="id not provided")
            self.assertEqual(card.name, name)
            return card
        except Exception as e:
            print(str(e))
            self.fail("Caught Exception adding card")

    def test40_add_card(self):
        name = "Testing from Python - no desc"
        card = self._add_card(name)

        self.assertIsNotNone(card.closed, msg="closed not provided")
        self.assertFalse(card.closed, msg="Card should not be closed")

        card2 = self._trello.get_card(card.id)
        self.assertEqual(card.name, card2.name)

    def test41_add_card_desc(self):
        name = "Testing from Python"
        description = "Description goes here"
        card = self._add_card(name, description)

        card.fetch()
        self.assertEqual(description, card.description)
        self.assertIsNotNone(card.url, msg="url not provided")
        self.assertIsNotNone(card.member_id)
        self.assertIsNotNone(card.short_id)
        self.assertIsNotNone(card.list_id)
        self.assertIsNotNone(card.comments)
        self.assertIsNotNone(card.checklists)
        self.assertIsInstance(card.created_date, datetime)

    def test42_add_card_with_comments_fetch(self):
        name = "Card with comments"
        comment = "Hello World!"
        card = self._add_card(name)
        card.comment(comment)
        card.fetch(True)

        self.assertEqual(len(card.comments), 1)
        self.assertEqual(card.comments[0]['data']['text'], comment)

    def test43_get_comments(self):
        name = "Card with comments 2"
        comment = "Hello World!"
        card = self._add_card(name)
        card.comment(comment)
        card._comments = card.get_comments()
        self.assertEqual(len(card.comments), 1)
        self.assertEqual(card.comments[0]['data']['text'], comment)

    def test44_attach_url_to_card(self):
        name = "Testing from Python - url"
        card = self._add_card(name)

        card.attach(name='lwn', url='http://lwn.net/')
        card.fetch()
        self.assertEqual(card.badges['attachments'], 1)
        card.delete()

    def test52_add_card_set_due(self):
        name = "Testing set due"
        card = self._add_card(name)

        # Set the due date to be 3 days from now
        today = datetime.today()
        day_detla = timedelta(3)
        due_date = today + day_detla
        card.set_due(due_date)
        expected_due_date = card.due
        # Refresh the due date from cloud
        card.fetch()
        actual_due_date = card.due
        self.assertEqual(expected_due_date[:8], actual_due_date[:8])

    def test_set_name(self):
        name = "Testing set card name"
        card = self._list.add_card('noname')
        card.set_name(name)
        self.assertEqual(card.name, name)

    def test_set_desc(self):
        card = self._list.add_card("Testing set card desc")
        description = "Description goes here"
        card.set_description(description)
        self.assertEqual(card.description, description)
        card.fetch()
        self.assertEqual(card.description, description)

    def test_set_closed(self):
        name = "Testing set card closed"
        card = self._list.add_card(name)
        self.assertFalse(card.closed)
        card.set_closed(True)
        self.assertTrue(card.closed)
        card.fetch()
        self.assertTrue(card.closed)

    def test81_resource_unavailable(self):
        self.assertRaises(ResourceUnavailable,
                          self._trello.get_card, '0dsfkjhsdf87342ed')


def suite():
    # tests = ['test01_list_boards', 'test10_board_attrs', 'test20_add_card']
    # return unittest.TestSuite(map(TrelloBoardTestCase, tests))
    return unittest.TestLoader().loadTestsFromTestCase(TrelloBoardTestCase)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
from trello.attachments import *
from trello.board import *
from trello.card import *
from trello.checklist import *
from trello.exceptions import *
from trello.label import *
from trello.member import *
from trello.organization import *
from trello.star import*
from trello.trelloclient import *
from trello.trellolist import *
from trello.webhook import *
from trello.util import *

# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4



# -*- coding: utf-8 -*-


class TrelloBase(object):
	def __init__(self):
		self.id = None

	def __hash__(self):
		class_name = type(self).__name__
		return hash(class_name) ^ hash(self.id)

	def __eq__(self, other):
		if isinstance(other, type(self)):
			return hash(self) == hash(other)
		raise NotImplementedError



# -*- coding: utf-8 -*-
from __future__ import with_statement, print_function, absolute_import

from trello import TrelloBase
from trello.compat import force_str


class Star(TrelloBase):
    """
    Class representing a Trello board star.
    """
    def __init__(self, star_id, board_id, position):
        super(Star, self).__init__()
        self.id = star_id
        self.board_id = board_id
        self.position = position

    @classmethod
    def from_json(cls, json_obj):
        """
        Deserialize the star json object to a Label object

        :board: the parent board the label is on
        :json_obj: the label json object
        """
        star = Star(star_id = json_obj['id'], board_id = json_obj['idBoard'], position = json_obj['pos'])
        return star

    @classmethod
    def from_json_list(cls, json_objs):
        return [cls.from_json(obj) for obj in json_objs]

    def __repr__(self):
        return force_str(u'<Star %s>' % self.id)

    def __eq__(self, other):
        return self.id == other.id

    def __ne__(self, other):
        return self.id != other.id



#!/usr/bin/python
from __future__ import with_statement, print_function
from datetime import datetime
import unittest
import os
from trello import TrelloClient, Board


class TrelloBoardTestCase(unittest.TestCase):
    """
    Tests for TrelloClient API. Note these test are in order to
    preserve dependencies, as an API integration cannot be tested
    independently.
    """

    @classmethod
    def setUpClass(cls):
        cls._trello = TrelloClient(os.environ['TRELLO_API_KEY'],
                                   token=os.environ['TRELLO_TOKEN'])
        for b in cls._trello.list_boards():
            if b.name == os.environ['TRELLO_TEST_BOARD_NAME']:
                cls._board = b
                break
        if not cls._board:
            cls.fail("Couldn't find test board")
        cls._list = cls._board.add_list(str(datetime.now()))

    def _add_card(self, name, description=None):
        try:
            card = self._list.add_card(name, description)
            self.assertIsNotNone(card, msg="card is None")
            self.assertIsNotNone(card.id, msg="id not provided")
            self.assertEqual(card.name, name)
            return card
        except Exception as e:
            print(str(e))
            self.fail("Caught Exception adding card")

    def _add_checklist(self, card, name, items=[], itemstates=None):
        checklist = card.add_checklist(name, items, itemstates)
        self.assertIsNotNone(checklist, msg="checklist is None")
        self.assertIsNotNone(checklist.id, msg="id not provided")
        self.assertEqual(checklist.name, name)
        return checklist

    def test_get_cards(self):
        # Let's ensure we have no cards in board
        for card in self._board.get_cards():
            card.delete()

        nb_cards = 3
        names = ["Card #" + str(i) for i in range(nb_cards)]
        for i in range(nb_cards):
            self._add_card(names[i])
        cards = self._board.get_cards()
        self.assertEqual(len(cards), nb_cards)
        self.assertEqual(len(cards), len(self._board.open_cards()))

        for card in cards:
            self.assertTrue(card.name in names, 'Unexpected card found')

        self.assertIsInstance(self._board.all_cards(), list)
        self.assertIsInstance(self._board.open_cards(), list)
        self.assertIsInstance(self._board.closed_cards(), list)
        self.assertIsInstance(self._board.visible_cards(), list)

    def test_fetch_action_limit(self):
        card = self._add_card('For action limit testing')
        card.set_closed(True)
        self._board.fetch_actions(action_filter='all', action_limit=2)
        actions = sorted(self._board.actions,key=lambda act: act['date'], reverse=True)
        self.assertEqual(len(actions), 2)
        self.assertEqual(actions[0]['type'], 'updateCard')
        self.assertFalse(actions[0]['data']['old']['closed'])
        self.assertEqual(actions[1]['type'], 'createCard')

    def test_fetch_action_filter(self):
        card = self._add_card('For action filter testing')
        card.set_closed(True) # This action will be skipped by filter
        self._board.fetch_actions(action_filter='createCard', action_limit=1)
        actions = self._board.actions
        self.assertEqual(len(actions), 1)
        self.assertEqual(actions[0]['type'], 'createCard')

    def test_delete_cards(self):
        self._add_card("card to be deleted")
        cards = self._board.open_cards()
        nb_open_cards = len(cards)
        for card in cards:
            card.delete()
        self._board.fetch_actions(action_filter='all', action_limit=nb_open_cards)
        self.assertEqual(len(self._board.actions), nb_open_cards)
        for action in self._board.actions:
            self.assertEqual(action['type'], 'deleteCard')

    def test_close_cards(self):
        nb_closed_cards = len(self._board.closed_cards())
        self._add_card("card to be closed")
        cards = self._board.open_cards()
        nb_open_cards = len(cards)
        for card in cards:
            card.set_closed(True)
        cards_after = self._board.closed_cards()
        nb_cards_after = len(cards_after)
        self.assertEqual(nb_cards_after, nb_closed_cards + nb_open_cards)


    def test_all_cards_reachable(self):
        if not len(self._board.open_cards()):
            self._add_card("an open card")
        if not len(self._board.closed_cards()):
            card = self._add_card("card to be closed")
            card.set_closed(True)
        self.assertEqual(len(self._board.all_cards()),
                          len(self._board.open_cards()) + len(self._board.closed_cards()))

    def test70_all_members(self):
        self.assertTrue(len(self._board.all_members()) > 0)

    def test71_normal_members(self):
        self.assertTrue(len(self._board.normal_members()) >= 0)

    def test72_admin_members(self):
        self.assertTrue(len(self._board.admin_members()) > 0)

    def test73_owner_members(self):
        members = self._board.owner_members()
        self.assertTrue(len(members) > 0)
        member = members[0].fetch()
        self.assertNotEqual(member.status, None)
        self.assertNotEqual(member.id, None)
        self.assertNotEqual(member.bio, None)
        self.assertNotEqual(member.url, None)
        self.assertNotEqual(member.username, None)
        self.assertNotEqual(member.full_name, None)
        self.assertNotEqual(member.initials, None)
        member2 = self._trello.get_member(member.id)
        self.assertEqual(member.username, member2.username)

    def test90_get_board(self):
        board = self._trello.get_board(self._board.id)
        self.assertEqual(self._board.name, board.name)

    def test100_add_board(self):
        test_board = self._trello.add_board("test_create_board")
        test_list = test_board.add_list("test_list")
        test_list.add_card("test_card")
        open_boards = self._trello.list_boards(board_filter="open")
        self.assertEqual(len([x for x in open_boards if x.name == "test_create_board"]), 1)

    def test110_copy_board(self):
        boards = self._trello.list_boards(board_filter="open")
        source_board = next( x for x in boards if x.name == "test_create_board")
        self._trello.add_board("copied_board", source_board=source_board)
        listed_boards = self._trello.list_boards(board_filter="open")
        copied_board = next(iter([x for x in listed_boards if x.name == "copied_board"]), None)
        self.assertIsNotNone(copied_board)
        open_lists = copied_board.open_lists()
        self.assertEqual(len(open_lists), 4) # default lists plus mine
        test_list = open_lists[0]
        self.assertEqual(len(test_list.list_cards()), 1)
        test_card = next ( iter([ x for x in test_list.list_cards() if x.name == "test_card"]), None )
        self.assertIsNotNone(test_card)

    def test120_close_board(self):
        boards = self._trello.list_boards(board_filter="open")
        open_count = len(boards)
        test_create_board = next( x for x in boards if x.name == "test_create_board") # type: Board
        copied_board = next( x for x in boards if x.name == "copied_board") # type: Board
        test_create_board.close()
        copied_board.close()
        still_open_boards = self._trello.list_boards(board_filter="open")
        still_open_count = len(still_open_boards)
        self.assertEqual(still_open_count, open_count - 2)

    def test130_get_checklists_board(self):
        chklists = self._board.get_checklists(cards = 'open')
        for chklst in chklists:
            chklst.delete()
        card = self._add_card('For checklist testing')
        chklist = self._add_checklist(card, "Test Checklist", items=["item1","item2"], itemstates = [True, False])
        new_chklists = self._board.get_checklists()
        test_chk = new_chklists[0]
        self.assertEqual(test_chk.name, "Test Checklist")
        self.assertEqual(test_chk.trello_card, card.id)
        self.assertEqual(len(new_chklists), 1)
        i1 = test_chk.items[0]
        i2 = test_chk.items[1]
        self.assertEqual(len(test_chk.items), 2)
        self.assertEqual(i1['name'], "item1")
        self.assertEqual(i1['state'], "complete")
        self.assertEqual(i2['name'], "item2")
        self.assertEqual(i2['state'], "incomplete")

    def test_last_activity(self):
        self.assertIsInstance(self._board.date_last_activity, datetime)
        self.assertIsInstance(self._board.get_last_activity(), datetime)

def suite():
    # tests = ['test01_list_boards', 'test10_board_attrs', 'test20_add_card']
    # return unittest.TestSuite(map(TrelloBoardTestCase, tests))
    return unittest.TestLoader().loadTestsFromTestCase(TrelloBoardTestCase)

if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import with_statement, print_function, absolute_import

from trello import TrelloBase
from trello.compat import force_str
from trello.member import Member


class Organization(TrelloBase):

    TIMEZONE = 'UTC'

    """
    Class representing an organization
    """
    def __init__(self, client, organization_id, name='', displayName=''):
        super(Organization, self).__init__()
        self.client = client
        self.id = organization_id
        self.name = name
        self.displayName = displayName

    @classmethod
    def from_json(cls, trello_client, json_obj):
        """
        Deserialize the board json object to a Organization object

        :trello_client: the trello client
        :json_obj: the board json object
        """
        organization = Organization(trello_client, json_obj['id'], name=json_obj['name'])
        organization.description = json_obj.get('desc', '')
        organization.url = json_obj['url']
        organization.displayName = json_obj['displayName']
        return organization

    def __repr__(self):
        return force_str(u'<Organization %s>' % self.name)

    def fetch(self):
        """Fetch all attributes for this organization"""
        json_obj = self.client.fetch_json('/organizations/' + self.id)
        self.name = json_obj['name']
        self.description = json_obj.get('desc', '')
        self.url = json_obj['url']

    def all_boards(self):
        """Returns all boards on this organization"""
        return self.get_boards('all')

    def get_boards(self, list_filter):
        """Get boards using filter

        :rtype: list of Board
        """
        from trello.board import Board
        json_obj = self.client.fetch_json(
            '/organizations/' + self.id + '/boards',
            query_params={'lists': 'none', 'filter': list_filter})
        return [Board.from_json(organization=self, json_obj=obj) for obj in json_obj]

    def get_board(self, field_name):
        """Get board

        :rtype: list of Board
        """
        from trello.board import Board
        json_obj = self.client.fetch_json(
            '/organizations/' + self.id + '/boards',
            query_params={'filter': 'open', 'fields': field_name})
        return [Board.from_json(organization=self, json_obj=obj) for obj in json_obj]

    def get_members(self):
        json_obj = self.client.fetch_json(
            '/organizations/' + self.id + '/members',
            query_params={'filter': 'all',
                          'fields': 'id,fullName,username,initials'})
        return [Member.from_json(trello_client=self.client, json_obj=obj) for obj in json_obj]

    def add_member(self, member, member_type="normal"):
        json_obj = self.client.fetch_json(
                '/organizations/{0}/members/{1}'.format(self.id, member.id),
                http_method='PUT',
                post_args={'idMember': member.id, "type": member_type},
        )
        return json_obj

    def remove_member(self, member):
        json_obj = self.client.fetch_json(
                '/organizations/{0}/members/{1}'.format(self.id, member.id),
                http_method='DELETE',
                post_args={'idMember': member.id},
        )
        return json_obj



# -*- coding: utf-8 -*-
from __future__ import with_statement, print_function, absolute_import

import time
import sys
from trello import TrelloBase
from trello.compat import force_str

is_python3 = sys.version_info.major == 3
if is_python3:
	unicode = str


class CustomFieldDefinition(TrelloBase):
	"""
	Class representing a Trello CustomFieldDefinition.
	"""
	def __init__(self, client, customFieldDefinition_id, name, field_type, list_options):
		super(CustomFieldDefinition, self).__init__()
		self.client = client
		self.id = customFieldDefinition_id
		self.name = name
		self.field_type = field_type
		self.list_options = list_options

	@classmethod
	def from_json(cls, board, json_obj):
		"""
		Deserialize the board's custom field json object to a CustomFieldDefinition object

		:param board: the parent board the custom field is on
		:param json_obj: the board's customField json object
		"""
		list_options = {}
		if json_obj['type'] == 'list':
			for option in json_obj['options']:
				list_options[option['id']] = option['value']['text']

		customFieldDefinition = CustomFieldDefinition(
			board.client,
			customFieldDefinition_id=json_obj['id'],
			name=json_obj['name'],
			field_type=json_obj['type'],
			list_options=list_options,
		)
		return customFieldDefinition

	@classmethod
	def from_json_list(cls, board, json_objs):
		return [cls.from_json(board, obj) for obj in json_objs]

	def __repr__(self):
		return force_str(u'<CustomFieldDefinition %s>' % (self.name,))


class CustomField(TrelloBase):
	"""
	Class representing a Trello CustomField.
	"""
	_type = ''

	def __init__(self, card, customField_id, customFieldDefinition_id, value):
		super(CustomField, self).__init__()
		self.client = card.client
		self.card = card
		self.id = customField_id
		self.definition_id = customFieldDefinition_id
		self._value = value

	@property
	def type(self):
		"""
		Returns the type of the custom field.
		:return: either 'list', 'checkbox', 'date', 'text' or 'number', str
		"""
		return self._type

	@property
	def name(self):
		"""
		Returns the user-readable name of the custom field by querying the
		custom field definitions.
		:return: the name of the custom field as str or None
		"""
		for definition in self.card.board.get_custom_field_definitions():
			if definition.id == self.definition_id:
				return definition.name
		return None

	@classmethod
	def from_json(cls, card, json_obj):
		"""
		Deserialize the custom field json object to a CustomField object

		:param card: the parent card the custom field is on
		:param json_obj: the customField json object
		"""
		raise Exception('Not Implemented')

	@classmethod
	def get_class(cls, board, json_obj):
		"""
		:param board: the board the custom field is used on
		:param json_obj: the customField json object
		:return: the python class that coresponds to the given custom field data
		"""
		definition_id = json_obj['idCustomField']
		for definition in board.get_custom_field_definitions():
			if definition.id == definition_id:
				return {
					'checkbox': CustomFieldCheckbox,
					'date': CustomFieldDate,
					'list': CustomFieldList,
					'number': CustomFieldNumber,
					'text': CustomFieldText,
				}[definition.field_type]
		return None

	@classmethod
	def from_json_list(cls, card, json_objs):
		return [cls.get_class(card.board, obj).from_json(card, obj) for obj in json_objs]

	def __repr__(self):
		return force_str(u'<CustomField%s %s=%r>' % (self.type.capitalize(), self.name, self.value))

	@property
	def value(self):
		raise Exception('Not Implemented')

	@value.setter
	def value(self, value):
		raise Exception('Not Implemented')


class CustomFieldText(CustomField):
	"""
	Class representing a Trello text custom field.
	"""
	_type = 'text'

	@classmethod
	def from_json(cls, card, json_obj):
		"""
		Deserialize the custom field json object to a CustomField object

		:card: the parent card the custom field is on
		:json_obj: the customField json object
		"""
		customField = cls(
			card,
			customField_id=json_obj['id'],
			customFieldDefinition_id=json_obj['idCustomField'],
			value=json_obj['value']['text'],
		)
		return customField

	@property
	def value(self):
		"""
		Returns the custom field value as unicode
		:return: the custom field value as unicode
		"""
		return self._value

	@value.setter
	def value(self, value):
		"""
		Sets the new value,
		:param value: the new value as unicode
		"""
		assert isinstance(value, unicode), "Given value is no unicode!"
		self.client.fetch_json(
			'/card/' + self.card.id + '/customField/' + self.definition_id + '/item',
			http_method='PUT',
			post_args={'value': { 'text': value, }, }, )
		self._value = value


class CustomFieldCheckbox(CustomField):
	_type = 'checkbox'

	@classmethod
	def from_json(cls, card, json_obj):
		"""
		Deserialize the custom field json object to a CustomField object

		:card: the parent board the custom field is on
		:json_obj: the customField json object
		"""
		customField = cls(
			card,
			customField_id=json_obj['id'],
			customFieldDefinition_id=json_obj['idCustomField'],
			value=(json_obj['value']['checked'] == u'true'),
		)
		return customField

	@property
	def value(self):
		"""
		Returns the custom field value as bool
		:return: the custom field value as bool
		"""
		return self._value

	@value.setter
	def value(self, value):
		"""
		Sets the new value,
		:param value: the new value as bool
		"""
		assert isinstance(value, bool), "Given value is no bool!"
		self.client.fetch_json(
			'/card/' + self.card.id + '/customField/' + self.definition_id + '/item',
			http_method='PUT',
			post_args={'value': { 'checked': u"true" if value else u"false", }, }, )
		self._value = value


class CustomFieldDate(CustomField):
	_type = 'date'

	@classmethod
	def from_json(cls, card, json_obj):
		"""
		Deserialize the custom field json object to a CustomField object

		:card: the parent card the custom field is on
		:json_obj: the customField json object
		"""
		customField = cls(
			card,
			customField_id=json_obj['id'],
			customFieldDefinition_id=json_obj['idCustomField'],
			value=json_obj['value']['date'],
		)
		return customField

	@property
	def value(self):
		"""
		Returns the custom field value as unicode in the format
		%Y-%m-%dT%H:%M:%S.000Z.
		:return: the custom field value as unicode
		"""
		return self._value

	@value.setter
	def value(self, value):
		"""
		Sets the new value,
		:param value: the new value as unicode in format %Y-%m-%dT%H:%M:%S.000Z
		"""
		assert isinstance(value, str) or isinstance(value, unicode), "Given value is no str or unicode!"
		time.strptime(value, '%Y-%m-%dT%H:%M:%S.000Z')
		self.client.fetch_json(
			'/card/' + self.card.id + '/customField/' + self.definition_id + '/item',
			http_method='PUT',
			post_args={'value': { 'date': value, }, }, )
		self._value = value


class CustomFieldList(CustomField):
	_type = 'list'

	@classmethod
	def from_json(cls, card, json_obj):
		"""
		Deserialize the custom field json object to a CustomField object

		:card: the parent card the custom field is on
		:json_obj: the customField json object
		"""
		customField = cls(
			card,
			customField_id=json_obj['id'],
			customFieldDefinition_id=json_obj['idCustomField'],
			value=json_obj['idValue'],
		)
		return customField

	def _id2str(self, _id):
		for definition in self.card.board.get_custom_field_definitions():
			if definition.id == self.definition_id:
				return definition.list_options.get(_id)
		raise Exception('Definition not found')

	def _str2id(self, text):
		for definition in self.card.board.get_custom_field_definitions():
			if definition.id == self.definition_id:
				for key, val in definition.list_options.items():
					if val == text:
						return key
				return None
		raise Exception('Definition not found')

	@property
	def value(self):
		"""
		Returns the custom field value as unicode.
		:return: the custom field value as unicode
		"""
		return self._id2str(self._value)

	@value.setter
	def value(self, value):
		"""
		Sets the new value, that must exist in custom field definition.
		:param value: the new value as unicode
		"""
		assert isinstance(value, str) or isinstance(value, unicode), "Given value is no str or unicode!"
		newvalue = self._str2id(value)
		assert newvalue is not None, "Unknown value has been specified!"
		self.client.fetch_json(
			'/card/' + self.card.id + '/customField/' + self.definition_id + '/item',
			http_method='PUT',
			post_args={'idValue': newvalue,}, )
		self._value = newvalue


class CustomFieldNumber(CustomField):
	_type = 'number'

	@classmethod
	def from_json(cls, card, json_obj):
		"""
		Deserialize the custom field json object to a CustomField object

		:card: the parent card the custom field is on
		:json_obj: the customField json object
		"""
		customField = cls(
			card,
			customField_id=json_obj['id'],
			customFieldDefinition_id=json_obj['idCustomField'],
			value=float(json_obj['value']['number']),
		)
		return customField

	@property
	def value(self):
		"""
		Returns the custom field value as unicode.
		:return: the custom field value as unicode
		"""
		return self._value

	@value.setter
	def value(self, value):
		"""
		Sets the new value,
		:param value: the new value as unicode
		"""
		assert isinstance(value, int) or isinstance(value, float), "Given value is no int or float!"
		self.client.fetch_json(
			'/card/' + self.card.id + '/customField/' + self.definition_id + '/item',
			http_method='PUT',
			post_args={'value': { 'number': str(value), }, }, )
		self._value = value



import sys

PY2 = sys.version_info < (3, 0)


def force_str(s, encoding='utf-8'):
    """
    Converts `s` to the `str` type, regardless of the Python
    version. This is useful for __repr__ return types, where a `str`
    (bytes) is expected in Python 2 and a `str` (unicode string) is
    expected in Python 3.
    """
    if PY2 and isinstance(s, unicode):  # noqa
        s = s.encode(encoding)
    return s



# -*- coding: utf-8 -*-
from __future__ import with_statement, print_function, absolute_import

from trello.base import TrelloBase
from trello.member import Member
from trello.card import Card
from trello.compat import force_str
from trello.trellolist import List
from trello.label import Label
from trello.checklist import Checklist
from trello.customfield import CustomFieldDefinition
from dateutil import parser as dateparser
from trello.powerup import PowerUp

class Board(TrelloBase):
	"""
	Class representing a Trello board. Board attributes are stored as normal
	Python attributes; access to all sub-objects, however, is always
	an API call (Lists, Cards).
	"""

	def __init__(self, client=None, board_id=None, organization=None, name=''):
		"""
		:trello: Reference to a Trello object
		:board_id: ID for the board

		Alternative Constructor

		:organization: reference to the parent organization
		:board_id: ID for this board

		"""
		super(Board, self).__init__()
		if organization is None:
			self.client = client
		else:
			self.organization = organization
			self.client = organization.client
		self.id = board_id
		self.name = name
		self._date_last_activity = None
		self.customFieldDefinitions = None

	@classmethod
	def from_json(cls, trello_client=None, organization=None, json_obj=None):
		"""
		Deserialize the board json object to a Board object

		:trello_client: the trello client
		:json_obj: the board json object

		Alternative contrustraction:

		Deserialize the board json object to a board object

		:organization: the organization object that the board belongs to
		:json_obj: the json board object
		"""
		if organization is None:
			board = Board(client=trello_client, board_id=json_obj['id'], name=json_obj['name'])
		else:
			board = Board(organization=organization, board_id=json_obj['id'], name=json_obj['name'])

		board.description = json_obj.get('desc', '')
		board.closed = json_obj['closed']
		board.url = json_obj['url']

		return board

	def __repr__(self):
		return force_str(u'<Board %s>' % self.name)

	def fetch(self):
		"""Fetch all attributes for this board"""
		json_obj = self.client.fetch_json('/boards/' + self.id)
		self.name = json_obj['name']
		self.description = json_obj.get('desc', '')
		self.closed = json_obj['closed']
		self.url = json_obj['url']
		self.customFieldDefinitions = None

	# Saves a Trello Board
	def save(self):
		json_obj = self.client.fetch_json(
				'/boards/',
				http_method='POST',
				post_args={'name': self.name, "desc": self.description, "defaultLists": False}, )
		# Set initial data from Trello
		self.from_json(json_obj=json_obj)
		self.id = json_obj["id"]

	def set_name(self, name):
		self.client.fetch_json(
				'/boards/{board_id}/name'.format(board_id=self.id),
				http_method='PUT',
				post_args={'value': name})
		self.name = name

	def set_description(self, desc):
		self.client.fetch_json(
				'/boards/{board_id}/desc'.format(board_id=self.id),
				http_method='PUT',
				post_args={'value': desc})
		self.description = desc

	def set_organization(self, desc):
		self.client.fetch_json(
				'/boards/{board_id}/idOrganization'.format(board_id=self.id),
				http_method='PUT',
				post_args={'value': desc})
		self.description = desc

	def close(self):
		self.client.fetch_json(
				'/boards/' + self.id + '/closed',
				http_method='PUT',
				post_args={'value': 'true', }, )
		self.closed = True

	def open(self):
		self.client.fetch_json(
				'/boards/' + self.id + '/closed',
				http_method='PUT',
				post_args={'value': 'false', }, )
		self.closed = False
		
	def delete(self):
		self.client.fetch_json(
				'/boards/' + self.id,
				http_method='DELETE', )

	def get_list(self, list_id):
		"""Get list

		:rtype: List
		"""
		obj = self.client.fetch_json('/lists/' + list_id)
		return List.from_json(board=self, json_obj=obj)

	def all_lists(self):
		"""Returns all lists on this board

		:rtype: list of List
		"""
		return self.get_lists('all')

	def open_lists(self):
		"""Returns all open lists on this board

		:rtype: list of List
		"""
		return self.get_lists('open')

	def closed_lists(self):
		"""Returns all closed lists on this board

		:rtype: list of List
		"""
		return self.get_lists('closed')

	def get_lists(self, list_filter):
		"""Get lists from filter

		:rtype: list of List
		"""
		# error checking
		json_obj = self.client.fetch_json(
				'/boards/' + self.id + '/lists',
				query_params={'cards': 'none', 'filter': list_filter})
		return [List.from_json(board=self, json_obj=obj) for obj in json_obj]

	def list_lists(self, list_filter='all'):
		"""Get lists from filter

		:rtype: list of List
		"""
		return self.get_lists(list_filter=list_filter)

	def get_custom_field_definitions(self):
		"""Get all custom field definitions for this board

		:rtype: list of CustomFieldDefinition
		"""
		if self.customFieldDefinitions is None:
			json_obj = self.client.fetch_json('/boards/' + self.id + '/customFields')
			self.customFieldDefinitions = CustomFieldDefinition.from_json_list(self, json_obj)
		return self.customFieldDefinitions

	def add_custom_field_definition(self, name, type, options=None, display_on_card=None, pos=None):
		"""Add a custom field definition to this board

		:name: name for the field
		:type: type of field: "checkbox", "list", "number", "text", "date"
		:options: list of options for field, only valid for "list" type
		:display_on_card: boolean whether this field should be shown on the front of cards 
		:pos: position of the list: "bottom", "top" or a positive number
		:return: the custom_field_definition
		:rtype: CustomFieldDefinition
		"""
		arguments = {'idModel': self.id, "modelType": "board", 'name': name, 'type': type}
		if options:
			arguments["options"] = options
		if display_on_card:
			arguments["display_cardFront"] = display_on_card
		if pos:
			arguments["pos"] = pos
		obj = self.client.fetch_json(
				'/customFields',
				http_method='POST',
				post_args=arguments, )
		return CustomFieldDefinition.from_json(board=self, json_obj=obj)

	def update_custom_field_definition(self, custom_field_definition_id, name=None, display_on_card=None, pos=None):
		"""Update a custom field definition on this board

		:custom_field_definition_id: the ID of the CustomFieldDefinition to update.
		:name: new name for the field
		:display_on_card: boolean whether this field should be shown on the front of cards 
		:pos: position of the list: "bottom", "top" or a positive number
		:return: the custom_field_definition
		:rtype: CustomFieldDefinition
		"""
		arguments = {}
		if name:
			arguments["name"] = name
		if not display_on_card is None:
			arguments["display/cardFront"] = u"true" if display_on_card else u"false"
		if pos:
			arguments["pos"] = pos
		
		json_obj = self.client.fetch_json(
			    '/customFields/{0}'.format(custom_field_definition_id),
			    http_method='PUT',
				post_args=arguments, )
		return json_obj

	def delete_custom_field_definition(self, custom_field_definition_id):
		"""Delete a custom_field_definition from this board

		:custom_field_definition_id: the ID of the CustomFieldDefinition to delete.
		:return: the custom field definition
		:rtype: json
		"""
		json_obj = self.client.fetch_json(
			    '/customFields/{0}'.format(custom_field_definition_id),
			    http_method='DELETE', )
		return json_obj
	
	def get_custom_field_list_options(self,custom_field_definition_id,values_only=False):
		"""Get custom field definition list options on this board

		:custom_field_definition_id: the ID of the CustomFieldDefinition.
		:values_only: Boolean to return only the values of the options, excluding color & position
		:return: the custom_field_definition
		:rtype: CustomFieldDefinition
		"""

		json_obj = self.client.fetch_json(
				'customFields/{0}/options'.format(custom_field_definition_id),
				http_method='GET',
				)
		if values_only:
			return [v for jo in json_obj for v in jo['value'].values()]
		return json_obj

	def add_custom_field_list_option(self,custom_field_definition_id,new_option):
		"""Update a custom field definition on this board

		:custom_field_definition_id: the ID of the CustomFieldDefinition to update.
		:new_option: The new option to add to the list
		:return: the custom_field_definition
		:rtype: CustomFieldDefinition
		"""

		json_obj = self.client.fetch_json(
				'customFields/{0}/options'.format(custom_field_definition_id),
				http_method='POST',
				post_args={'value': {'text':new_option},
						}, )
		return json_obj
	
	def get_custom_field_list_option(self,custom_field_definition_id,option_id):
		"""Get a specific custom field definition list option on this board

		:custom_field_definition_id: the ID of the CustomFieldDefinition.
		:option_id: the ID of the option
		:return: the custom_field_definition
		:rtype: CustomFieldDefinition
		"""

		json_obj = self.client.fetch_json(
				'customFields/{0}/options/{1}'.format(custom_field_definition_id,option_id),
				http_method='GET',
				)
		# if values_only:
		# 	return [v for jo in json_obj for v in jo['value'].values()]
		return json_obj

	def delete_custom_field_list_option(self,custom_field_definition_id,option_id):
		"""DELETE a specific custom field definition list option on this board

		:custom_field_definition_id: the ID of the CustomFieldDefinition.
		:option_id: the ID of the option
		:return: the custom_field_definition
		:rtype: CustomFieldDefinition
		"""

		json_obj = self.client.fetch_json(
				'customFields/{0}/options/{1}'.format(custom_field_definition_id,option_id),
				http_method='DELETE',
				)
		# if values_only:
		# 	return [v for jo in json_obj for v in jo['value'].values()]
		return json_obj

	def get_labels(self, fields='all', limit=50):
		"""Get label

		:rtype: list of Label
		"""
		json_obj = self.client.fetch_json(
				'/boards/' + self.id + '/labels',
				query_params={'fields': fields, 'limit': limit})
		return Label.from_json_list(self, json_obj)
	
	def get_label(self, label_id):
		"""
		:label_id: str label id

		:rtype: instance of Label
		"""
		json_obj = self.client.fetch_json(
			'/boards/' + self.id + '/labels/' + label_id
		)
		
		return Label.from_json(self, json_obj)

	def get_checklists(self, cards='all'):
		"""Get checklists

		:rtype: list of Checklist
		"""
		checklists = []
		json_obj = self.client.fetch_json(
				'/boards/' + self.id + '/checklists',
				query_params={'cards': cards})
		json_obj = sorted(json_obj, key=lambda checklist: checklist['pos'])
		for cl in json_obj:
			checklists.append(Checklist(self.client, cl, trello_card=cl.get('idCard')))
		return checklists

	def add_list(self, name, pos=None):
		"""Add a list to this board

		:name: name for the list
		:pos: position of the list: "bottom", "top" or a positive number
		:return: the list
		:rtype: List
		"""
		arguments = {'name': name, 'idBoard': self.id}
		if pos:
			arguments["pos"] = pos
		obj = self.client.fetch_json(
				'/lists',
				http_method='POST',
				post_args=arguments, )
		return List.from_json(board=self, json_obj=obj)

	def add_label(self, name, color):
		"""Add a label to this board

		:name: name of the label
		:color: the color, either green, yellow, orange
			red, purple, blue, sky, lime, pink, or black
		:return: the label
		:rtype: Label
		"""
		obj = self.client.fetch_json(
				'/labels',
				http_method='POST',
				post_args={'name': name, 'idBoard': self.id, 'color': color}, )
		return Label.from_json(board=self, json_obj=obj)

	def delete_label(self, label_id):
		"""Delete a label from this board

		:label_id: the ID of the label to delete.
		:return: the label
		:rtype: json
		"""
		json_obj = self.client.fetch_json(
			    '/labels/{0}'.format(label_id),
			    http_method='DELETE',
		    	post_args={'id': label_id}, )
		return json_obj

	def all_cards(self, custom_field_items='true'):
		"""Returns all cards on this board

		:rtype: list of Card
		"""
		filters = {
			'filter': 'all',
			'fields': 'all',
			'customFieldItems': custom_field_items
		}
		return self.get_cards(filters)

	def open_cards(self, custom_field_items='true'):
		"""Returns all open cards on this board

		:rtype: list of Card
		"""
		filters = {
			'filter': 'open',
			'fields': 'all',
			'customFieldItems': custom_field_items
		}
		return self.get_cards(filters)

	def closed_cards(self, custom_field_items='true'):
		"""Returns all closed cards on this board

		:rtype: list of Card
		"""
		filters = {
			'filter': 'closed',
			'fields': 'all',
			'customFieldItems': custom_field_items
		}
		return self.get_cards(filters)

	def visible_cards(self, custom_field_items='true'):
		"""Returns all visible cards on this board

		:rtype: list of Card
		"""
		filters = {
			'filter': 'visible',
			'fields': 'all',
			'customFieldItems': custom_field_items
		}
		return self.get_cards(filters)

	def get_cards(self, filters=None, card_filter=""):
		"""
		:filters: dict containing query parameters. Eg. {'fields': 'all'}
		:card_filter: filters on card status ('open', 'closed', 'all')

		More info on card queries:
		https://trello.com/docs/api/board/index.html#get-1-boards-board-id-cards

		:rtype: list of Card
		"""
		json_obj = self.client.fetch_json(
				'/boards/' + self.id + '/cards/' + card_filter,
				query_params=filters
		)

		return list([Card.from_json(self, json) for json in json_obj])
	
	def get_card(self, card_id):
		"""
		:card_id: str card id.

		More info on card queries:
		https://developer.atlassian.com/cloud/trello/rest/api-group-boards/#api-boards-id-cards-idcard-get

		:rtype: instance of Card
		"""
		json_obj = self.client.fetch_json(
				'/boards/' + self.id + '/cards/' + card_id
		)

		return Card.from_json(self, json_obj)

	def all_members(self):
		"""Returns all members on this board

		:rtype: list of Member
		"""
		filters = {
			'filter': 'all',
			'fields': 'all'
		}
		return self.get_members(filters)

	def normal_members(self):
		"""Returns all normal members on this board

		:rtype: list of Member
		"""
		filters = {
			'filter': 'normal',
			'fields': 'all'
		}
		return self.get_members(filters)

	def admin_members(self):
		"""Returns all admin members on this board

		:rtype: list of Member
		"""
		filters = {
			'filter': 'admins',
			'fields': 'all'
		}
		return self.get_members(filters)

	def owner_members(self):
		"""Returns all owner members on this board

		:rtype: list of Member
		"""
		filters = {
			'filter': 'owners',
			'fields': 'all'
		}
		return self.get_members(filters)

	def get_members(self, filters=None):
		"""Get members with filter

		:filters: dict containing query parameters.
			Eg. {'fields': 'all', 'filter': 'admins'}

		More info on possible filters:
		https://developers.trello.com/advanced-reference/board#get-1-boards-board-id-members

		:rtype: list of Member
		"""
		json_obj = self.client.fetch_json(
				'/boards/' + self.id + '/members',
				query_params=filters)
		members = list()
		for obj in json_obj:
			m = Member.from_json(self.client, obj)
			members.append(m)

		return members

	# Add a member to a board
	def add_member(self, member, member_type="normal"):
		json_obj = self.client.fetch_json(
				'/boards/{0}/members/{1}'.format(self.id, member.id),
				http_method='PUT',
				post_args={'idMember': member.id, "type": member_type},
		)
		return json_obj

	# Removes an existing member of a board
	def remove_member(self, member):
		json_obj = self.client.fetch_json(
				'/boards/{0}/members/{1}'.format(self.id, member.id),
				http_method='DELETE',
				post_args={'idMember': member.id},
		)
		return json_obj

	def fetch_actions(self, action_filter, action_limit=50, before=None, since=None):
		"""Returns all actions that conform to the given filters.

		:action_filter: str of possible actions separated by comma
			ie. 'createCard,updateCard'
		:action_limit: int of max items returned
		:before: datetime obj
		:since: datetime obj

		More info on action filter values:
		https://developers.trello.com/advanced-reference/board#get-1-boards-board-id-actions

		:rtype: json list of past actions
		"""
		query_params = {'filter': action_filter, 'limit': action_limit}

		if since:
			query_params["since"] = since

		if before:
			query_params["before"] = before

		json_obj = self.client.fetch_json('/boards/' + self.id + '/actions',
				query_params=query_params)

		self.actions = json_obj
		return self.actions

	@property
	def date_last_activity(self):
		if self._date_last_activity is None:
			self._date_last_activity = self.get_last_activity()
		return self._date_last_activity

	def get_last_activity(self):
		"""Return the date of the last action done on the board.

		:rtype: datetime.datetime
		"""
		json_obj = self.client.fetch_json(
                '/boards/{0}/dateLastActivity'.format(self.id))
		if json_obj['_value']:
			return dateparser.parse(json_obj['_value'])
	def get_power_ups(self, board_id=None, name='', filters=None):
		'''
		List the Power-Ups on a board
		:filters: defaults to enabled
		valid values: enabled, available
		'''
		arguments = {}
		if board_id is None:
			board_id = self.id
		if filters:
			if not filters in ("enabled", "available"):
				filers = "enabled"
			arguments['filter'] = filters
		json_obj = self.client.fetch_json(
			'/boards/' + board_id +'/plugins',
			 http_method='GET',post_args=arguments, )
		return list([PowerUp.from_json(self, json_obj=json) for json in json_obj])
	def enable_power_up(self, powerup_id,board_id=None):
		from warnings import warn
		warn("Atlassian has DEPRECATED enabling power ups! enable_power_up will be updated once an alternative becomes available",DeprecationWarning,stacklevel=2)
		if board_id is None:
			board_id = self.id
		json_obj = self.client.fetch_json(
			'boards/' + board_id + '/boardPlugins',
			http_method='POST',
			post_args={'idPlugin':powerup_id},)
		return json_obj
	def disable_power_up(self, powerup_id, board_id=None):
		from warnings import warn
		warn("Atlassian has DEPRECATED disabling power ups! disable_power_up will be updated once an alternative becomes available",DeprecationWarning,stacklevel=2)
		if board_id is None:
			board_id = self.id
		json_obj = self.client.fetch_json(
			'boards/' + board_id + '/boardPlugins' + powerup_id,
			http_method='DELETE')
		return json_obj
	def get_enabled_power_ups(self, board_id=None, name='',):
		'''
		List the enabled Power-Ups on a board
		'''
		if board_id is None:
			board_id = self.id
		json_obj = self.client.fetch_json(
			'boards/' + board_id + '/boardPlugins',
			http_method='GET')
		print(json_obj)
		return list([PowerUp.from_json(self, json_obj=json) for json in json_obj])




# -*- coding: utf-8 -*-
from __future__ import with_statement, print_function, absolute_import


class WebHook(object):
    """Class representing a Trello webhook."""

    def __init__(self, client, token, hook_id=None, desc=None, id_model=None,
                 callback_url=None, active=False):
        self.id = hook_id
        self.desc = desc
        self.id_model = id_model
        self.callback_url = callback_url
        self.active = active
        self.client = client
        self.token = token

    def delete(self):
        """Removes this webhook from Trello"""
        self.client.fetch_json(
            '/webhooks/%s' % self.id,
            http_method='DELETE')



# -*- coding: utf-8 -*-
from __future__ import with_statement, print_function, absolute_import

import datetime
from operator import itemgetter

import pytz
from dateutil import parser as dateparser

from trello import TrelloBase
from trello.attachments import Attachments
from trello.checklist import Checklist
from trello.compat import force_str
from trello.label import Label
from trello.organization import Organization
from trello.customfield import CustomField, CustomFieldText, CustomFieldCheckbox, CustomFieldNumber, CustomFieldDate, CustomFieldList


class Card(TrelloBase):
    """
    Class representing a Trello card. Card attributes are stored on
    the object

    https://developers.trello.com/advanced-reference/card
    """

    @property
    def short_url(self):
        return self.shortUrl

    @property
    def member_id(self):
        return self.idMembers

    @property
    def short_id(self):
        return self.idShort

    @property
    def list_id(self):
        return self.idList

    @property
    def board_id(self):
        return self.idBoard

    @property
    def description(self):
        return self.desc

    @property
    def date_last_activity(self):
        return self.dateLastActivity

    @property
    def labels(self):
        return self._labels if self._labels else []

    @property
    def custom_fields(self):
        """
        Lazily loads and returns the custom fields
        """
        if self.customFields is None:
            self.customFields = self.fetch_custom_fields()
        return self.customFields

    @property
    def comments(self):
        """
        Lazily loads and returns the comments
        """
        if self._comments is None:
            self._comments = self.fetch_comments()
        return self._comments

    @property
    def checklists(self):
        """
        Lazily loads and returns the checklists
        """
        if self._checklists is None:
            self._checklists = self.fetch_checklists()
        return self._checklists

    @property
    def plugin_data(self):
        """
        Lazily loads and returns the plugin data
        """
        if self._plugin_data is None:
            self._plugin_data = self.fetch_plugin_data()
        return self._plugin_data

    @property
    def attachments(self):
        """
        Lazily loads and returns the attachments
        """
        if self._attachments is None:
            self._attachments = self.fetch_attachments()
        return self._attachments

    def __init__(self, parent, card_id, name=''):
        """
        :parent: reference to the parent trello list
        :card_id: ID for this card
        """
        super(Card, self).__init__()
        if isinstance(parent, List):
            self.trello_list = parent
            self.board = parent.board
        else:
            self.board = parent

        self.client = parent.client
        self.id = card_id
        self.name = name

        self.customFields = None
        self._checklists = None
        self._comments = None
        self._plugin_data = None
        self._attachments = None
        self._labels = None
        self._json_obj = None

    @classmethod
    def from_json(cls, parent, json_obj):
        """
        Deserialize the card json object to a Card object

        :parent: the list object that the card belongs to
        :json_obj: json object

        :rtype: Card
        """
        if 'id' not in json_obj:
            raise Exception("key 'id' is not in json_obj")
        card = cls(parent,
                   json_obj['id'],
                   name=json_obj['name'])
        card._json_obj = json_obj
        card.desc = json_obj.get('desc', '')
        card.due = json_obj.get('due', '')
        card.is_due_complete = json_obj['dueComplete']
        card.closed = json_obj['closed']
        card.url = json_obj['url']
        card.pos = json_obj['pos']
        card.shortUrl = json_obj['shortUrl']
        card.idMembers = json_obj['idMembers']
        card.member_ids = json_obj['idMembers']
        card.idLabels = json_obj['idLabels']
        card.idBoard = json_obj['idBoard']
        card.idList = json_obj['idList']
        card.idShort = json_obj['idShort']
        card.badges = json_obj['badges']
        card.customFields = card.fetch_custom_fields(json_obj=json_obj)
        card.countCheckItems = json_obj['badges']['checkItems']
        card.countCheckLists = len(json_obj['idChecklists'])
        card._labels = Label.from_json_list(card.board, json_obj['labels'])
        card.dateLastActivity = dateparser.parse(json_obj['dateLastActivity'])
        if "attachments" in json_obj:
            card._attachments = []
            for attachment_json in json_obj["attachments"]:
                card._attachments.append(attachment_json)
        if 'actions' in json_obj:
            card.actions = json_obj['actions']
        return card

    def __repr__(self):
        return force_str(u'<Card %s>' % self.name)

    def fetch(self, eager=True):
        """
        Fetch all attributes for this card

        :param eager: If eager, comments, checklists and attachments will be fetched immediately, otherwise on demand
        """
        json_obj = self.client.fetch_json(
            '/cards/' + self.id,
            query_params={'badges': False, 'customFieldItems': 'true'})
        self.id = json_obj['id']
        self.name = json_obj['name']
        self.desc = json_obj.get('desc', '')
        self.closed = json_obj['closed']
        self.url = json_obj['url']
        self.shortUrl = json_obj['shortUrl']
        self.idMembers = json_obj['idMembers']
        self.idShort = json_obj['idShort']
        self.idList = json_obj['idList']
        self.idBoard = json_obj['idBoard']
        self.idLabels = json_obj['idLabels']
        self._labels = Label.from_json_list(self.board, json_obj['labels'])
        self.badges = json_obj['badges']
        self.pos = json_obj['pos']
        if json_obj.get('due', ''):
            self.due = json_obj.get('due', '')
        else:
            self.due = ''
        self.dateLastActivity = dateparser.parse(json_obj['dateLastActivity'])

        self._customFields = self.fetch_custom_fields(json_obj=json_obj)
        self._plugin_data = self.fetch_plugin_data() if eager else None
        self._checklists = self.fetch_checklists() if eager else None
        self._comments = self.fetch_comments() if eager else None
        self._attachments = self.fetch_attachments() if eager else None

    def fetch_custom_fields(self, json_obj=None):
        """
        Fetch current set of custom fields from card or json_obj.
        """
        if json_obj is None:
            json_obj = self.client.fetch_json(
                '/cards/' + self.id,
                query_params={'badges': False, 'customFieldItems': 'true'})
        return CustomField.from_json_list(
            self, json_obj.get('customFieldItems', {}))

    def fetch_comments(self, force=False, limit=None):
        comments = []

        if (force is True) or (self.badges['comments'] > 0):
            query_params = {'filter': 'commentCard,copyCommentCard'}
            if limit is not None:
                query_params['limit'] = limit
            comments = self.client.fetch_json(
                '/cards/' + self.id + '/actions',
                query_params=query_params)
            return sorted(comments, key=lambda comment: comment['date'])
        return comments

    def get_list(self):
        obj = self.client.fetch_json('/lists/' + self.idList)
        return List.from_json(board=self, json_obj=obj)

    def get_comments(self):
        """Alias for fetch_comments for backward compatibility.
        Always contact server
        """
        return self.fetch_comments(force=True)

    def fetch_checklists(self):

        if self.countCheckLists == 0:
            return []

        checklists = []
        json_obj = self.client.fetch_json(
            '/cards/' + self.id + '/checklists', )
        # Thanks https://github.com/HuffAndPuff for noticing checklist
        # were not sorted
        json_obj = sorted(json_obj, key=lambda checklist: checklist['pos'])
        for cl in json_obj:
            checklists.append(Checklist(self.client, cl,
                                        trello_card=self.id))
        return checklists

    def fetch_plugin_data(self):
        items = self.client.fetch_json(
            '/cards/' + self.id + '/pluginData')
        return items

    def fetch_attachments(self, force=False):
        if (force is True) or (self.badges['attachments'] > 0):
            items = self.client.fetch_json(
                '/cards/' + self.id + '/attachments',
                query_params={'filter':'false'})
            return items
        return []

    def get_attachments(self):
        return [Attachments.from_json(attachments_json) for attachments_json in self.fetch_attachments(force=True)]

    def fetch_actions(self, action_filter='createCard', since=None, before=None, action_limit=50):
        """
        Fetch actions for this card can give more argv to action_filter,
        split for ',' json_obj is list
        """
        query_params={'filter': action_filter, 'limit': action_limit}
        if since:
            query_params["since"] = since

        if before:
            query_params["before"] = before

        json_obj = self.client.fetch_json(
            '/cards/' + self.id + '/actions',
            query_params=query_params)

        self.actions = json_obj
        return self.actions

    def attriExp(self, multiple):
        """
            Provides the option to explore what comes from trello
            :multiple is one of the attributes of GET /1/cards/[card id or shortlink]/actions
        """
        self.fetch_actions(multiple)
        return self.actions

    @staticmethod
    def _movement_as_triplet(source_list, destination_list, movement_datetime):
        return [source_list["name"], destination_list["name"], movement_datetime]

    @staticmethod
    def _movement_as_dict(source_list, destination_list, movement_datetime):
        _movement = {
            "source": source_list,
            "destination": destination_list,
            "datetime": movement_datetime,
        }
        return _movement

    def _list_movements(self, movement_function, filter_by_date_interval=None):
        """
        Returns the list of movements of this card.
        The list of movements is in descending date and time order. First movement is the closest one to now.
        Its structure is a list of dicts where the lists are "source" and "destination" and both are also dicts.
        Date and time of the movement is in key "datetime" as a datetime object.
        :param movement_function: function that returns a representation of the movement.
        :param filter_by_date_interval: Date interval used to filter card movements to return. Optional
        :return: list with the movements.
        """

        action_since = None if not filter_by_date_interval else filter_by_date_interval[0]
        action_before = None if not filter_by_date_interval else filter_by_date_interval[1]
        if not hasattr(self, "actions") or self.actions is None:
            self.fetch_actions('updateCard:idList,', action_since, action_before)

        movements = []

        for idx in self.actions:
            date_str = idx['date']
            movement_datetime = dateparser.parse(date_str)
            try:
                source_list = idx['data']['listBefore']
                destination_list = idx['data']['listAfter']
            except KeyError:
                continue
            movement = movement_function(source_list, destination_list, movement_datetime)
            movements.append(movement)

        return movements

    def listCardMove_date(self):
        """Will return the history of transitions of a card from one list to
        another. The lower the index the more recent the historical item.

        It returns a list of lists. The sublists are triplets of
        starting list, ending list and when the transition occurred.
        """
        return self._list_movements(movement_function=Card._movement_as_triplet)

    def list_movements(self, list_cmp=None, filter_by_date_interval=None):
        """Will return the history of transitions of a card from one list to
        another. The lower the index the more recent the historical item.

        It returns a list of dicts in date and time descending order (the
        first movement is the most recent).
        Dicts are of the form source:
        <listobj> destination: <listobj> datetime: <datetimeobj>

        :param: list_cmp Comparison function between lists. For list_cmp(a, b) returns -1 if list a is greater that list b. Returns 1 otherwise.
        :param: filter_by_date_interval: pair of two dates (two strings in YYYY-MM-DD format) to filter card movements by date.
        """
        movement_as_dict_function = Card._movement_as_dict
        if list_cmp:
            def movement_as_dict_function(_source_list, _destination_list, _movement_datetime):
                _movement = Card._movement_as_dict(_source_list, _destination_list, _movement_datetime)
                _source_list_id = _source_list["id"]
                _destination_list_id = _destination_list["id"]
                _movement["moving_forward"] = list_cmp(_source_list_id, _destination_list_id) > 0
                return _movement

        return self._list_movements(movement_function=movement_as_dict_function, filter_by_date_interval=filter_by_date_interval)

    def get_stats_by_list(self, lists, list_cmp=None, done_list=None, time_unit="seconds", card_movements_filter=None):
        """Gets several stats about the card by each list of the board:
        - time: The time that the card has been in each column in seconds (minutes or hours).
        - forward_moves: How many times this card has been the source of a forward movement.
        - backward_moves: How many times this card has been the source of a backward movement.

        Returns a dict where the key is list id and value is a dict with keys
        time, forward_moves and backward_moves.

        :param lists: list of board lists.
        :param list_cmp: function that compares two lists a,b given id_a, id_b. If b is in a forward position returns 1 else -1.
        :param time_unit: default to seconds. Allow specifying time in "minutes" or "hours".
        :param done_list: Column that implies that the task is done. If present, time measurement will be stopped if is current task list.
        :param card_movements_filter: Pair of two dates (two strings in YYYY-MM-DD format) that will filter the movements of the card. Optional.
        :return: dict of the form {list_id: {time:<time card was in that list>, forward_moves: <number>, backward_moves: <number> }}
        """
        tz = pytz.timezone(Organization.TIMEZONE)

        # Conversion of units
        seconds_to_time_unit = lambda time: time
        if time_unit == "minutes":
            seconds_to_time_unit = lambda time: time / 60.0
        elif time_unit == "hours":
            seconds_to_time_unit = lambda time: time / 3660.0

        # Creation datetime of the card
        creation_datetime = self.created_date

        #  Time in seconds stores the seconds that our card lives in a column
        stats_by_list = {list_.id: {"time":0, "forward_moves":0, "backward_moves":0} for list_ in lists}

        #  Last action date, used to compute the time the card spends between changes
        # of columns
        last_action_datetime = creation_datetime

        # Changes of columns of our card
        # Using list comparison function (if present) to check list position and, hence,
        # if the card movement was forward or backwards
        changes = self.list_movements(list_cmp, card_movements_filter)

        #  If there are no changes in the card, all its life has been in its creation list
        if len(changes) == 0:
            card_life_time = seconds_to_time_unit((datetime.datetime.now(tz) - last_action_datetime).total_seconds())
            stats_by_list[self.idList]["time"] += card_life_time

        else:
            # Changes in card are ordered to get the dates in order
            last_list = None

            ordered_changes = sorted(changes, key=itemgetter("datetime"))
            # For each arrival to a list, its datetime will be used to compute
            # the time this card is in that destination list
            for change in ordered_changes:
                source_list = change["source"]
                destination_list = change["destination"]
                change_datetime = change["datetime"]

                # For each column the total number of seconds this card is computed
                source_list_id = source_list["id"]

                time_from_last_list_change = seconds_to_time_unit((change_datetime - last_action_datetime).total_seconds())

                # Our last action has been this change
                last_action_datetime = change_datetime

                # In case the source or destination list is not a list of this board, ignore them
                if source_list_id not in stats_by_list:
                    continue

                stats_by_list[source_list_id]["time"] += time_from_last_list_change

                # Count if the change is to move forward or backwards
                if "moving_forward" in change:
                    if change["moving_forward"]:
                        stats_by_list[source_list_id]["forward_moves"] += 1
                    else:
                        stats_by_list[source_list_id]["backward_moves"] += 1

                # Store the last list
                last_list = destination_list

            # Adding the number of seconds the card has been in its last column (until now)
            # only if the last column is not "Done" column
            if done_list and last_list and last_list["id"] and last_list["id"] in stats_by_list and\
                last_list["id"] != done_list.id:
                time_card_has_spent_in_list_until_now = seconds_to_time_unit((datetime.datetime.now(tz) - last_action_datetime).total_seconds())
                stats_by_list[last_list["id"]]["time"] += time_card_has_spent_in_list_until_now

        return stats_by_list

    @property
    def latestCardMove_date(self):
        """Returns the date of the last card transition"""
        self.fetch_actions('updateCard:idList')
        if self.actions is None or len(self.actions) == 0:
            return None
        date_str = self.actions[0]['date']
        return dateparser.parse(date_str)

    @property
    def created_date(self):
        """Will return the creation date of the card.

        WARNING: if the card was create via convertion of a checklist item
                it fails. attriExp('convertToCardFromCheckItem') allows to
                test for the condition.
        """
        if not hasattr(self, "creation_date"):
            localtz = pytz.timezone(Organization.TIMEZONE)
            self.creation_date = localtz.localize(datetime.datetime.fromtimestamp(int(self.id[0: 8], 16)))
        return self.creation_date

    @property
    def card_created_date(self):
        """Will return the creation date of the card.

        NOTE: This will return the date the card was created, even if it
        was created on another board. The created_date() above actually just
        returns the first activity and has the issue described in the warning.

        The first 8 characters of the card id is a hexadecimal number.
        Converted to a decimal from hexadecimal, the timestamp is an Unix
        timestamp (the number of seconds that have elapsed since January 1,
        1970 midnight UTC. See
        http://help.trello.com/article/759-getting-the-time-a-card-or-board-was-created
        """
        unix_time = int(self.id[:8], 16)

        return datetime.datetime.fromtimestamp(unix_time)

    @property
    def due_date(self):
        return dateparser.parse(self.due) if self.due else ''

    def set_name(self, new_name):
        """Update the name on the card to :new_name:

        :new_name: str
        """
        self._set_remote_attribute('name', new_name)
        self.name = new_name

    def set_description(self, description):
        self._set_remote_attribute('desc', description)
        self.desc = description

    def set_due(self, due):
        """Set the due time for the card

        :due: a datetime object
        """
        datestr = due.isoformat()
        self._set_remote_attribute('due', datestr)
        self.due = datestr

    def set_start(self, start):
        """Set the start time for the card

        :start: a datetime object
        """
        datestr = start.isoformat()
        self._set_remote_attribute('start', datestr)
        self.start = datestr
        
    def set_reminder(self, reminder):
        """Set a reminder time for the card

        :reminder: total number of minutes before the due date as an int
        """
        # datestr = reminder.isoformat()
        if isinstance(reminder,(float,str)):
            reminder = int(float(reminder))
        self._set_remote_attribute('dueReminder', reminder)
        self.reminder = reminder

    def set_due_complete(self):
        """Set due complete

        :return: None
        """
        self._set_due_complete(True)

    def remove_due_complete(self):
        """Remove due complete

        :return: None
        """
        self._set_due_complete(False)


    def remove_due(self):
        """
        Remove the due datetime of this card.
        """
        self._set_remote_attribute('due', None)
        self.due = ''

    def set_pos(self, pos):
        """
        Update card position in list

        :pos: 'top', 'bottom' or int
        """
        self._set_remote_attribute('pos', pos)
        self.pos = pos

    def set_custom_field(self, value, custom_field):
        """Update card custom field

        Arguments:
            value {[str, int, date, bool]} -- Value depending on the type of custom_field
            custom_field {custom field object} -- Custom Field Object (board.get_custom_field_definitions()[0])

        """
        if custom_field.field_type in ['text', 'number', 'date', 'checked']:
            if value == "":
                post_args = {'value': ""}
            else:
                post_args = {'value': {str(custom_field.field_type): value}}
        else:
            if value == "":
                list_field_id = ""
            else:
                list_field_id = [
                    x for x, y in custom_field.list_options.items() if y == value][0]
            post_args = {'idValue': list_field_id}

        self.client.fetch_json(
            '/card/' + self.id + '/customField/' + custom_field.id + '/item',
            http_method='PUT',
            post_args=post_args)

    def set_closed(self, closed):
        self._set_remote_attribute('closed', closed)
        self.closed = closed

    def delete_comment(self, comment):
        # Delete this comment permanently
        self.client.fetch_json(
            '/cards/' + self.id + '/actions/' + comment['id'] + '/comments',
            http_method='DELETE')

    def delete(self):
        # Delete this card permanently
        self.client.fetch_json(
            '/cards/' + self.id,
            http_method='DELETE')

    def assign(self, member_id):
        self.client.fetch_json(
            '/cards/' + self.id + '/members',
            http_method='POST',
            post_args={'value': member_id})

    def unassign(self, member_id):
        self.client.fetch_json(
            '/cards/' + self.id + '/idMembers/' + member_id,
            http_method='DELETE')

    def subscribe(self):
        self.client.fetch_json(
            '/cards/' + self.id + '/subscribed',
            http_method='PUT',
            post_args={'value': True})

    def comment(self, comment_text):
        """Add a comment to a card.

        :comment_text: str
        """
        comment_data = self.client.fetch_json(
            '/cards/' + self.id + '/actions/comments',
            http_method='POST',
            post_args={'text': comment_text})
        return comment_data

    def update_comment(self, comment_id, comment_text):
        """Update a comment."""
        comment_data = self.client.fetch_json(
            '/actions/' + comment_id,
            http_method='PUT',
            post_args={'text': comment_text}
        )
        return comment_data

    def add_label(self, label):
        self.client.fetch_json(
            '/cards/' + self.id + '/idLabels',
            http_method='POST',
            post_args={'value': label.id})

    def create_label(self, name, color):
        self.client.fetch_json(
            "/cards/" + self.id + "/labels",
            http_method='POST',
            post_args={"name": name, "color": color})

    def remove_label(self, label):
        self.client.fetch_json(
            '/cards/' + self.id + '/idLabels/' + label.id,
            http_method='DELETE')

    def add_member(self, member):
        self.client.fetch_json(
            '/cards/' + self.id + '/idMembers',
            http_method='POST',
            post_args={'value': member.id})

    def remove_member(self, member):
        self.client.fetch_json(
            '/cards/' + self.id + '/idMembers/' + member.id,
            http_method='DELETE')

    def attach(self, name=None, mimeType=None, file=None, url=None, setCover=None):
        """
        Add an attachment to the card. The attachment can be either a
        file or a url. Setting the name and/or mime type is optional.
        :param name: The name of the attachment
        :param mimeType: mime type for the attachement
        :param file: a file-like, binary object that supports read()
        :param url: a URL pointing to the resource to be attached
        :param cover: boolean
        """
        if (file and url) or (not file and not url):
            raise Exception('Please provide either a file or url, and not both!')

        kwargs = {}
        if file:
            kwargs['files'] = dict(file=(name, file, mimeType))
        else:
            kwargs['name'] = name
            kwargs['mimeType'] = mimeType
            kwargs['url'] = url
            kwargs['setCover'] = setCover

        return self._post_remote_data('attachments', **kwargs)

    def remove_attachment(self, attachment_id):
        """
        Remove attachment from card
        :param attachment_id: Attachment id
        :return: None
        """
        self.client.fetch_json(
            '/cards/' + self.id + '/attachments/' + attachment_id,
            http_method='DELETE')

    def change_pos(self, position):
        self.client.fetch_json(
            '/cards/' + self.id + '/pos',
            http_method='PUT',
            post_args={'value': position})

    def change_list(self, list_id):
        self.client.fetch_json(
            '/cards/' + self.id + '/idList',
            http_method='PUT',
            post_args={'value': list_id})

    def change_board(self, board_id, list_id=None):
        args = {'value': board_id}
        if list_id is not None:
            args['idList'] = list_id
        self.client.fetch_json(
            '/cards/' + self.id + '/idBoard',
            http_method='PUT',
            post_args=args)

    def add_checklist(self, title, items, itemstates=None):
        """Add a checklist to this card

        :title: title of the checklist
        :items: a list of the item names
        :itemstates: a list of the state (True/False) of each item
        :return: the checklist
        """
        if itemstates is None:
            itemstates = []

        json_obj = self.client.fetch_json(
            '/cards/' + self.id + '/checklists',
            http_method='POST',
            post_args={'name': title}, )

        cl = Checklist(self.client, json_obj, trello_card=self.id)
        for i, name in enumerate(items):
            try:
                checked = itemstates[i]
            except IndexError:
                checked = False
            cl.add_checklist_item(name, checked)

        self.fetch()
        return cl

    def _set_due_complete(self, is_complete):
        """Set due is complete or not complete

        https://developers.trello.com/advanced-reference/card#put-1-cards-card-id-or-shortlink-dueComplete
        :param is_complete: boolean
        :return: None
        """
        self.client.fetch_json('/cards/' + self.id + '/dueComplete',
                               http_method='PUT',
                               post_args={'value': is_complete})

    def _set_remote_attribute(self, attribute, value):
        self.client.fetch_json(
            '/cards/' + self.id + '/' + attribute,
            http_method='PUT',
            post_args={'value': value}, )

    def _post_remote_data(self, attribute, files=None, **kwargs):
        return self.client.fetch_json(
            '/cards/' + self.id + '/' + attribute,
            http_method='POST',
            files=files,
            post_args=kwargs)

    def get_custom_field_by_name(self, cf_name):
        """
        Returns existing custom field by name or creates a new one.
        """
        for cf in self.customFields:
            if cf.name == cf_name:
                return cf

        cf_class = None
        cf_def_id = None
        for definition in self.board.get_custom_field_definitions():
            if definition.name == cf_name:
                cf_def_id = definition.id
                cf_class = {
                    'checkbox': CustomFieldCheckbox,
                    'date': CustomFieldDate,
                    'list': CustomFieldList,
                    'number': CustomFieldNumber,
                    'text': CustomFieldText,
                }.get(definition.field_type)
        if cf_class is None:
            raise ValueError('Unknown custom field name specified ({})'.format(cf_name))
        return cf_class(self, 'unknown', cf_def_id, '')

from trello.trellolist import List



# -*- coding: utf-8 -*-
import sys

from trello.util import create_oauth_token


def main():
    try:
        command = sys.argv[1]
    except IndexError:
        return

    if command == "oauth":
        create_oauth_token()


if __name__ == "__main__":
    sys.exit(main())

# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4



#!/usr/bin/python
from __future__ import with_statement, print_function
from datetime import datetime
import unittest
import os
from trello import TrelloClient


class TrelloChecklistTestCase(unittest.TestCase):
    """
    Tests for TrelloClient API. Note these test are in order to
    preserve dependencies, as an API integration cannot be tested
    independently.
    """

    @classmethod
    def setUpClass(cls):
        cls._trello = TrelloClient(os.environ['TRELLO_API_KEY'],
                                   token=os.environ['TRELLO_TOKEN'])
        for b in cls._trello.list_boards():
            if b.name == os.environ['TRELLO_TEST_BOARD_NAME']:
                cls._board = b
                break
        if not cls._board:
            cls.fail("Couldn't find test board")
        cls._list = cls._board.add_list(str(datetime.now()))

    def _add_card(self, name, description=None):
        try:
            card = self._list.add_card(name, description)
            self.assertIsNotNone(card, msg="card is None")
            self.assertIsNotNone(card.id, msg="id not provided")
            self.assertEqual(card.name, name)
            return card
        except Exception as e:
            print(str(e))
            self.fail("Caught Exception adding card")

    def _add_checklist(self, card, name, items=[]):
        checklist = card.add_checklist(name, items)
        self.assertIsNotNone(checklist, msg="checklist is None")
        self.assertIsNotNone(checklist.id, msg="id not provided")
        self.assertEqual(checklist.name, name)
        return checklist

    def test_delete_checklist(self):
        name = "Card with checklist, test deletion"
        card = self._list.add_card(name)
        card.fetch(eager=True)
        checklist = self._add_checklist(card, 'ChecklistsToDelete', ['item1', 'item2'])
        checklist.delete()
        card._checklists = card.fetch_checklists()
        self.assertEqual(len(card.checklists), 0)

    def test_delete_checklist_remaining(self):
        name = "Card with checklist, test deletion"
        card = self._list.add_card(name)
        card.fetch(eager=True)

        name = 'ChecklistsToDelete'
        checklist_delete = self._add_checklist(card, name, ['item1', 'item2'])

        name_keep = 'ChecklistsToKeep'
        self._add_checklist(card, name_keep, [])
        checklist_delete.delete()
        card._checklists = card.fetch_checklists()
        self.assertEqual(len(card.checklists), 1)
        self.assertEqual(card.checklists[0].name, name_keep)

    def test_checklist_rename(self):
        name = "Testing checklist rename"
        description = "Description goes here"
        card = self._list.add_card(name, description)

        name = 'ToBeRenamed'
        new_name = "Renamed"
        checklist = self._add_checklist(card, name, ['item1', 'item2'])
        checklist.rename(new_name)
        self.assertEqual(checklist.name, new_name)
        card._checklists = card.fetch_checklists()
        self.assertEqual(len(card.checklists), 1)
        self.assertEqual(card.checklists[0].name, new_name)

    def test_delete_checklist_item(self):
        name = "Testing checklist item delete"
        card = self._list.add_card(name, "Description goes here")

        name = 'Checklist'
        checklist = self._add_checklist(card, name, ['item1', 'item2'])
        checklist.delete_checklist_item('item2')

        checklists = card.fetch_checklists()
        self.assertEqual(len(checklists[0].items), 1)
        self.assertEqual(checklists[0].items[0]['name'], 'item1')

    def test_clear_checklist(self):
        name = "Testing checklist clear"
        card = self._list.add_card(name, "Description goes here")

        name = 'Checklist'
        checklist = self._add_checklist(card, name, ['item1', 'item2', 'item3'])
        checklist.clear()

        checklists = card.fetch_checklists()
        self.assertEqual(len(checklists[0].items), 0)


def suite():
    # tests = ['test01_list_boards', 'test10_board_attrs', 'test20_add_card']
    # return unittest.TestSuite(map(TrelloBoardTestCase, tests))
    return unittest.TestLoader().loadTestsFromTestCase(TrelloChecklistTestCase)

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(
    name="py-trello",
    version="0.19.0",

    description='Python wrapper around the Trello API',
    long_description=open('README.rst').read(),
    author='Richard Kolkovich',
    author_email='richard@sigil.org',
    url='https://trello.com/board/py-trello/4f145d87b2f9f15d6d027b53',
    download_url='https://github.com/sarumont/py-trello',
    keywords='python',
    license='BSD License',
    classifiers=[
        "Development Status :: 4 - Beta",
            'Intended Audience :: Developers',
            'License :: OSI Approved :: BSD License',
            'Operating System :: OS Independent',
            'Programming Language :: Python',
            'Programming Language :: Python :: 2',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.3',
    ],
    install_requires=["requests", "requests-oauthlib >= 0.4.1", "python-dateutil", "pytz"],
    packages=find_packages(),
    include_package_data=True,
)
# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4



# -*- coding: utf-8 -*-
from __future__ import with_statement, print_function, absolute_import

from trello import TrelloBase
from trello.compat import force_str


class Member(TrelloBase):
    """
    Class representing a Trello member.
    """

    def __init__(self, client, member_id, full_name=''):
        super(Member, self).__init__()
        self.client = client
        self.id = member_id
        self.full_name = full_name

    def __repr__(self):
        return force_str(u'<Member %s>' % self.id)

    def fetch(self):
        """Fetch all attributes for this member"""
        json_obj = self.client.fetch_json(
            '/members/' + self.id,
            query_params={'badges': False})
        self.email = json_obj['email']
        self.status = json_obj['status']
        self.id = json_obj.get('id', '')
        self.bio = json_obj.get('bio', '')
        self.url = json_obj.get('url', '')
        self.username = json_obj['username']
        self.full_name = json_obj['fullName']
        self.initials = json_obj['initials']
        self.member_type = json_obj.get('memberType', '')
        self.avatar_url = json_obj['avatarUrl']
        return self

    def fetch_comments(self):
        if self.badges['comments'] > 0:
            comments = self.client.fetch_json(
                '/members/' + self.id + '/actions',
                query_params={'filter': 'commentCard'})
            return sorted(comments, key=lambda comment: comment['date'])
        return []

    def fetch_cards(self):
        """ Fetches all the cards for this member """
        cards = self.client.fetch_json(
            '/members/' + self.id + '/cards',
            query_params={'filter': 'visible'})
        return sorted(cards, key=lambda card: card['dateLastActivity'])

    def fetch_notifications(self, filters = []):
        """ Fetches all the notifications for this member """
        notifications = self.client.fetch_json(
            '/members/' + self.id + '/notifications',
            query_params={'filter': ",".join(filters)})
        return sorted(notifications, key=lambda notification: notification['date'])

    def get_boards(self, list_filter):
        """Get boards using filter

        :rtype: list of Board
        """
        from trello.board import Board
        from trello.organization import Organization
        json_obj = self.client.fetch_json(
            '/members/' + self.id + '/boards',
            query_params={'lists': 'none', 'filter': list_filter})
        organizations = {obj['idOrganization']: self.client.get_organization(obj['idOrganization']) for obj in json_obj if obj['idOrganization']}
        return [Board.from_json(trello_client=self.client, organization=organizations.get(obj['idOrganization']), json_obj=obj) for obj in json_obj]

    @classmethod
    def from_json(cls, trello_client, json_obj):
        """
        Deserialize the organization json object to a member object

        :trello_client: the trello client
        :json_obj: the member json object
        """
        member = Member(trello_client, json_obj['id'], full_name=json_obj['fullName'])
        member.email = json_obj.get('email', '')
        member.status = json_obj.get('status', '')
        member.bio = json_obj.get('bio', '')
        member.url = json_obj.get('url', '')
        member.username = json_obj.get('username', '')
        member.initials = json_obj.get('initials', '')
        member.member_type = json_obj.get('memberType', '')
        member.avatar_url = json_obj.get('avatarUrl', '')
        return member



# -*- coding: utf-8 -*-
from __future__ import with_statement, print_function, absolute_import
import json
import requests
from requests_oauthlib import OAuth1
from trello.board import Board
from trello.card import Card
from trello.trellolist import List
from trello.organization import Organization
from trello.member import Member
from trello.webhook import WebHook
from trello.exceptions import *
from trello.label import Label
from trello.star import Star

try:
    # PyOpenSSL works around some issues in python ssl modules
    # In particular in python < 2.7.9 and python < 3.2
    # It is not a hard requirement, so it's not listed in requirements.txt
    # More info https://urllib3.readthedocs.org/en/latest/security.html#insecureplatformwarning
    import urllib3.contrib.pyopenssl

    urllib3.contrib.pyopenssl.inject_into_urllib3()
except:
    pass


class TrelloClient(object):
    """ Base class for Trello API access """

    def __init__(self, api_key, api_secret=None, token=None, token_secret=None, http_service=requests, proxies={}):
        """
        Constructor

        :api_key: API key generated at https://trello.com/1/appKey/generate
        :api_secret: the secret component of api_key
        :token: OAuth token generated by the user in
                    trello.util.create_oauth_token
        :token_secret: the OAuth client secret for the given OAuth token
        """

        # client key and secret for oauth1 session
        if token or token_secret:
            self.oauth = OAuth1(client_key=api_key, client_secret=api_secret,
                                resource_owner_key=token, resource_owner_secret=token_secret)
        else:
            self.oauth = None

        self.proxies = proxies
        self.public_only = token is None
        self.api_key = api_key
        self.api_secret = api_secret
        self.resource_owner_key = token
        self.resource_owner_secret = token_secret
        self.http_service = http_service

    def info_for_all_boards(self, actions):
        """
        Use this if you want to retrieve info for all your boards in one swoop
        """
        if self.public_only:
            return None
        else:
            json_obj = self.fetch_json(
                '/members/me/boards/all',
                query_params={'actions': actions})
            self.all_info = json_obj

    def logout(self):
        """Log out of Trello."""
        # TODO: This function.
        raise NotImplementedError()

    def list_boards(self, board_filter="all"):
        """
        Returns all boards for your Trello user

        :return: a list of Python objects representing the Trello boards.
        :rtype: list of Board

        Each board has the following noteworthy attributes:
            - id: the board's identifier
            - name: Name of the board
            - desc: Description of the board (optional - may be missing from the
                    returned JSON)
            - closed: Boolean representing whether this board is closed or not
            - url: URL to the board
        """
        json_obj = self.fetch_json('/members/me/boards/?filter=%s' % board_filter)
        return [Board.from_json(self, json_obj=obj) for obj in json_obj]

    def list_organizations(self):
        """
        Returns all organizations for your Trello user

        :return: a list of Python objects representing the Trello organizations.
        :rtype: list of Organization

        Each organization has the following noteworthy attributes:
            - id: the organization's identifier
            - name: Name of the organization
            - desc: Description of the organization (optional - may be missing from the
                    returned JSON)
            - closed: Boolean representing whether this organization is closed or not
            - url: URL to the organization
        """
        json_obj = self.fetch_json('members/me/organizations')
        return [Organization.from_json(self, obj) for obj in json_obj]

    def get_organization(self, organization_id):
        """Get organization

        :rtype: Organization
        """
        obj = self.fetch_json('/organizations/' + organization_id)

        return Organization.from_json(self, obj)

    def get_board(self, board_id):
        """Get board

        :rtype: Board
        """
        obj = self.fetch_json('/boards/' + board_id)
        return Board.from_json(self, json_obj=obj)

    def add_organization(self, organization_name, description=None, name=None):
        """Create organization
        :param organization_name: Name of the organization to create
        :param description: Optional Description of the organization
        :param name: Unique name of the organization, like a slug
        :rtype: Organization
        """
        post_args = {'displayName': organization_name}
        if description is not None:
            post_args['desc'] = description
        if name is not None:
            post_args['name'] = name

        obj = self.fetch_json('/organizations', http_method='POST', post_args=post_args)
        return Organization.from_json(self, json_obj=obj)

    def add_board(self, board_name, source_board=None, organization_id=None, permission_level='private',
                  default_lists=True):
        """Create board
        :param board_name: Name of the board to create
        :param source_board: Optional Board to copy
        :param permission_level: Permission level, defaults to private
        :rtype: Board
        """
        post_args = {'name': board_name, 'prefs_permissionLevel': permission_level}
        if source_board is not None:
            post_args['idBoardSource'] = source_board.id
        if organization_id is not None:
            post_args['idOrganization'] = organization_id
        if not default_lists:
            post_args['defaultLists'] = False

        obj = self.fetch_json('/boards', http_method='POST',
                              post_args=post_args)
        return Board.from_json(self, json_obj=obj)

    def get_member(self, member_id):
        """Get member

        :rtype: Member
        """
        return Member(self, member_id).fetch()

    def get_card(self, card_id):
        """Get card

        :rtype: Card
        """
        card_json = self.fetch_json('/cards/' + card_id, query_params={'customFieldItems': 'true'})
        list_json = self.fetch_json('/lists/' + card_json['idList'])
        board = self.get_board(card_json['idBoard'])
        return Card.from_json(List.from_json(board, list_json), card_json)

    def get_list(self, list_id):
        """Get list

        :rtype: List
        """
        list_json = self.fetch_json('/lists/' + list_id)
        board = self.get_board(list_json['idBoard'])
        return List.from_json(board, list_json)

    def get_label(self, label_id, board_id):
        """Get Label

        Requires the parent board id the label is on

        :rtype: Label
        """
        board = self.get_board(board_id)
        label_json = self.fetch_json('/labels/' + label_id)
        return Label.from_json(board, label_json)

    def fetch_json(
            self,
            uri_path,
            http_method='GET',
            headers=None,
            query_params=None,
            post_args=None,
            files=None):
        """ Fetch some JSON from Trello """

        # explicit values here to avoid mutable default values
        if headers is None:
            headers = {}
        if query_params is None:
            query_params = {}
        if post_args is None:
            post_args = {}

        # if files specified, we don't want any data
        data = None
        if files is None:
            data = json.dumps(post_args)

        # set content type and accept headers to handle JSON
        if http_method in ("POST", "PUT", "DELETE") and not files:
            headers['Content-Type'] = 'application/json; charset=utf-8'

        headers['Accept'] = 'application/json'

        # construct the full URL without query parameters
        if uri_path[0] == '/':
            uri_path = uri_path[1:]
        url = 'https://api.trello.com/1/%s' % uri_path

        if self.oauth is None:
            query_params['key'] = self.api_key
            query_params['token'] = self.api_secret

        # perform the HTTP requests, if possible uses OAuth authentication
        response = self.http_service.request(http_method, url, params=query_params,
                                             headers=headers, data=data,
                                             auth=self.oauth, files=files,
                                             proxies=self.proxies)

        if response.status_code == 401:
            raise Unauthorized("%s at %s" % (response.text, url), response)
        if response.status_code != 200:
            raise ResourceUnavailable("%s at %s" % (response.text, url), response)

        return response.json()

    def list_hooks(self, token=None):
        """
        Returns a list of all hooks associated with a specific token. If you don't pass in a token,
        it tries to use the token associated with the TrelloClient object (if it exists)
        """
        token = token or self.resource_owner_key or self.api_secret

        if token is None:
            raise TokenError("You need to pass an auth token in to list hooks.")
        else:
            url = "/tokens/%s/webhooks" % token
            return self._existing_hook_objs(self.fetch_json(url), token)

    def _existing_hook_objs(self, hooks, token):
        """
        Given a list of hook dicts passed from list_hooks, creates
        the hook objects
        """
        all_hooks = []
        for hook in hooks:
            new_hook = WebHook(self, token, hook['id'], hook['description'],
                               hook['idModel'],
                               hook['callbackURL'], hook['active'])
            all_hooks.append(new_hook)
        return all_hooks

    def create_hook(self, callback_url, id_model, desc=None, token=None):
        """
        Creates a new webhook. Returns the WebHook object created.

        There seems to be some sort of bug that makes you unable to create a
        hook using httplib2, so I'm using urllib2 for that instead.
        """
        token = token or self.resource_owner_key or self.api_secret

        if token is None:
            raise TokenError("You need to pass an auth token in to create a hook.")

        url = "https://trello.com/1/tokens/%s/webhooks/" % token
        data = {'callbackURL': callback_url, 'idModel': id_model,
                'description': desc}

        if self.api_key is not None:
            data.update({'key': self.api_key})

        response = self.http_service.post(url, data=data, auth=self.oauth, proxies=self.proxies)

        if response.status_code == 200:
            hook_id = response.json()['id']
            return WebHook(self, token, hook_id, desc, id_model, callback_url, True)
        else:
            raise Exception("Webhook creating failed: %s" % response.text)

    def search(self, query, partial_match=False, models=[],
               board_ids=[], org_ids=[], card_ids=[], cards_limit=10):
        """
        Search trello given a query string.

        :param str query: A query string up to 16K characters
        :param bool partial_match: True means that trello will look for
                content that starts with any of the words in your query.
        :param list models: Comma-separated list of types of objects to search.
                This can be 'actions', 'boards', 'cards', 'members',
                or 'organizations'.  The default is 'all' models.
        :param list board_ids: Comma-separated list of boards to limit search
        :param org_ids: Comma-separated list of organizations to limit search
        :param card_ids: Comma-separated list of cards to limit search
        :param cards_limit: The maximum number of cards to return (up to 1000)

        :return: All objects matching the search criterial.  These can
            be Cards, Boards, Organizations, and Members.  The attributes
            of the objects in the results are minimal; the user must call
            the fetch method on the resulting objects to get a full set
            of attributes populated.
        :rtype list:
        """

        query_params = {'query': query}

        if partial_match:
            query_params['partial'] = 'true'

        # Limit search to one or more object types
        if models:
            query_params['modelTypes'] = models

        # Limit search to a particular subset of objects
        if board_ids:
            query_params['idBoards'] = board_ids
        if org_ids:
            query_params['idOrganizations'] = org_ids
        if card_ids:
            query_params['idCards'] = card_ids
        query_params['cards_limit'] = cards_limit

        # Request result fields required to instantiate class objects
        query_params['board_fields'] = ['name,url,desc,closed']
        query_params['member_fields'] = ['fullName,initials,username']
        query_params['organization_fields'] = ['name,url,desc']

        json_obj = self.fetch_json('/search', query_params=query_params)
        if not json_obj:
            return []

        results = []
        board_cache = {}

        for board_json in json_obj.get('boards', []):
            # Cache board objects
            if board_json['id'] not in board_cache:
                board_cache[board_json['id']] = Board.from_json(
                    self, json_obj=board_json)
            results.append(board_cache[board_json['id']])

        for card_json in json_obj.get('cards', []):
            # Cache board objects
            if card_json['idBoard'] not in board_cache:
                board_cache[card_json['idBoard']] = Board(
                    self, card_json['idBoard'])
                # Fetch the board attributes as the Board object created
                # from the card initially result lacks a URL and name.
                # This Board will be stored in Card.parent
                board_cache[card_json['idBoard']].fetch()
            results.append(Card.from_json(board_cache[card_json['idBoard']],
                                          card_json))

        for member_json in json_obj.get('members', []):
            results.append(Member.from_json(self, member_json))

        for org_json in json_obj.get('organizations', []):
            org = Organization.from_json(self, org_json)
            results.append(org)

        return results

    def list_stars(self):
        """
        Returns all boardStars for your Trello user

        :return: a list of Python objects representing the Trello board stars.
        :rtype: list of Board Stars

        Each board has the following noteworthy attributes:
            - id: the board star's identifier
            - idBoard: ID of starred board
            - pos: position of the board star
        """
        json_obj = self.fetch_json('/members/me/boardStars')
        return [Star.from_json(json_obj=obj) for obj in json_obj]

    def add_star(self, board_id, position="bottom"):
        """Create a star
        :param board_iid: Id of the board to star
        :param position: Optional position of the board star
        :rtype: Star
        """
        post_args = {'idBoard': board_id, 'pos': position}
        obj = self.fetch_json('members/me/boardStars', http_method='POST',
                              post_args=post_args)
        return Star.from_json(json_obj=obj)

    def delete_star(self, star):
        """Deletes a star
        :param board_iid: Id of the board to star
        :param position: Optional position of the board star
        :rtype: Star
        """
        self.fetch_json('members/me/boardStars/{}'.format(star.id), http_method='DELETE')
        return star
